;+
; Mission     : ASO-S/HXI
; Name        : hxi_read_fits
; Purpose     : read single HXI_ql or HXI_lev1 data file
; Inputs      :
;       filename  - input data file
;       ftype     - 'ql' or 'sp'
; Outputs     :
;       return data structure
;
;Written: 
; 2021/01/05, Fanxiaoyu Xia
;Modifications:
; 2021/03/29, Fanxiaoyu Xia, modified the way of checing extname.
; 2021/04/27, Fanxiaoyu Xia, added flag informations for quicklook structure.
; 2022/12/07, Yang Su, modified the code due to the changes of lev1 and levQ1 data
; 2023/03/09, Yang Su, add bkg_edge; modified the code due to the changes of lev1 and levQ1 data
; 2023/11/08, Yang Su, add 'thinthick' and 'bkg_thinthick'
; 2023/12/29, Weichen, Zhentong Li, add keyword tcalib, time calibration for hxi,set to default

function hxi_read_fits, filename, ftype =ftype, tcalib= tcalib
  
  checkvar, tcalib, 1

  if ~file_exist(filename) then begin
    message, 'Argument file is undefined.',/cont
    return,-1
  endif
  
  fits_open, filename, fcb, /no_abort, message=err_msg
  if err_msg ne '' then return, -1
  fits_close, fcb
  
  case ftype of 
    'ql': begin
      primary=mrdfits(filename,0,header0,/silent)
  
      headerstr=fitshead2struct(header0)
      date_ref=headerstr.date_ref
      
      ;;n_ext=1 , info about time,  mode, flag, etc.
      info_ext = where_arr(strlowcase(fcb.extname), strlowcase('hxi ql info'))
      tinfo =mrdfits(filename,info_ext[0],/silent)
      qlfits_times=anytim(date_ref)+tinfo.time
      ;time correction, 2023/12/29; 2024/01/08 chenwei
       if keyword_set(tcalib) AND  qlfits_times[0] lt anytim('01-Jan-24')  then qlfits_times +=  hxi_timelag_calib(qlfits_times)/1000.
      qlfits_expt= tinfo.EXPTIME ;yang
   
      ntimes=n_elements(qlfits_times)
      
      ; n_ext =5 ;yang , 221116
      flag_ext   = where_arr(strlowcase(fcb.extname), strlowcase('flags'))
      flag_info = mrdfits(filename,flag_ext[0],/silent) 
      qlfits_flagt = anytim(date_ref)+flag_info.time
      ;time correction, 2023/12/30 Yang Su; 2024/01/08, Wei Chen
      if keyword_set(tcalib) AND  qlfits_flagt[0] lt anytim('01-Jan-24')  then qlfits_flagt +=  hxi_timelag_calib(qlfits_flagt)/1000.
      ntimes_flags = n_elements(qlfits_flagt)
      
      qlfits_flags=intarr(6, ntimes_flags)
      ind0=where(flag_info.mode eq 2)
      qlfits_flags[0, ind0]=1    ;flag_SAF
      ind1=where(flag_info.flags eq 8)
      qlfits_flags[1, ind1]=1    ;flag_NT
      ind2=where(flag_info.mode eq 8)
      qlfits_flags[2, ind2]=1    ;flag_SAA
      ind3=where(flag_info.mode eq 4)
      qlfits_flags[3, ind3]=1    ;flag_CAL
      ind4=where(flag_info.flags eq 2)
      qlfits_flags[4, ind4]=1    ;flag_FL
      ind5= where(flag_info.flags eq 4)
      qlfits_flags[5, ind5]=1    ;flag_LST
  
      ;;n_ext=2 info about energy band
      ebound_ext= where_arr(strlowcase(fcb.extname), strlowcase('ebounds'))
      eneinfo= mrdfits(filename,ebound_ext[0],h2,/silent)
      e1=eneinfo.en_min
      e2=eneinfo.en_max
      qlfits_enebins=[[e1],[e2]]
      
      ;;n_ext=3 info about counts data
      ct_ext= where_arr(strlowcase(fcb.extname), strlowcase('crate'))
      dinfo1=mrdfits(filename,ct_ext[0],/silent)
      qlfits_data1=dinfo1.CTS_thinthick  ;231108,YS
      qlfits_data2=dinfo1.CTS_THIN
      qlfits_data3=dinfo1.cts_bkg_center  ;221204,yang su
      qlfits_data4=dinfo1.cts_bkg_edge   ;230309,yang
      qlfits_bkg_thinthick = dinfo1.cts_bkg_thinthick ;231230, YS
      qlfits_data5=total(dinfo1.CTS_TOT_MODU,1)
      qlfits_data6=total(dinfo1.CTS_TOT_SPEC,1)
      
      qlfits_livet_thinthick = dinfo1.livetime_thinthick ;231108,YS
      qlfits_livet_thin =  dinfo1.livetime_thin 
      qlfits_livet_bg =  dinfo1.livetime_bg 
   
      ;;n_ext=4 info about monitor
      mt_ext= where_arr(strlowcase(fcb.extname), strlowcase('monitor'))
      dinfo2=mrdfits(filename,mt_ext[0],/silent)
  
      qlfits_data7=dinfo2.pointing
      qlfits_data8=dinfo2.roll_ang
      qlfits_data9=dinfo2.temper
      qlfits_data10=dinfo2.hv
      qlfits_data11=dinfo2.xyz
  
      ;dinfo1
  ;    LIVETIME_THICK  FLOAT           3.99777
  ;    LIVETIME_THIN   FLOAT           3.99784
  ;    LIVETIME_BG     FLOAT           3.99776
      
      qlstr={qlfits_times:qlfits_times, qlfits_expt: qlfits_expt, qlfits_enebins:qlfits_enebins,$
        qlfits_livet_thinthick:qlfits_livet_thinthick, qlfits_livet_thin:qlfits_livet_thin,qlfits_livet_bg:qlfits_livet_bg, $ 
        qlfits_data1:qlfits_data1, qlfits_data2:qlfits_data2, qlfits_data3:qlfits_data3, $
        qlfits_data4:qlfits_data4, qlfits_data5:qlfits_data5, qlfits_data6:qlfits_data6, $
        qlfits_data7:qlfits_data7, qlfits_data8:qlfits_data8, qlfits_data9:qlfits_data9, $
        qlfits_data10:qlfits_data10,qlfits_data11:qlfits_data11,qlfits_flagt:qlfits_flagt, qlfits_flags:qlfits_flags,$
        qlfits_bkg_thinthick:qlfits_bkg_thinthick} ;231230,YS
  
      return, qlstr
    end
    
    'sp': begin
    info_ext = where_arr(strlowcase(fcb.extname), strlowcase('info'))
    primary=mrdfits(filename,0,header0,/silent)
    headerstr=fitshead2struct(header0)
    date_ref=headerstr.date_ref
    tinfo =mrdfits(filename,info_ext[0],/silent)
    spfits_times=anytim(date_ref)+tinfo.time
    ;time correction, 2023/12/29; 2024/01/08 chenwei
    if keyword_set(tcalib)  AND  spfits_times[0] lt anytim('01-Jan-24')   then spfits_times +=  hxi_timelag_calib(spfits_times)/1000.
    
    spfits_expt= tinfo.EXPTIME ;yang
  
    ebound_ext= where_arr(strlowcase(fcb.extname), strlowcase('ebounds'))
    eneinfo= mrdfits(filename, ebound_ext[0],h2,/silent)
    e1=eneinfo.en_min
    e2=eneinfo.en_max
    spfits_spenebins=[[e1],[e2]]
    nenergy=fxpar(h2,'naxis2')  
    
    spec_ext= where_arr(strlowcase(fcb.extname), strlowcase('hxi_spec'))
    dinfo1=mrdfits(filename,3,/silent)
    spfits_spdata1=dinfo1.counts
    
    dinfo2=mrdfits(filename,spec_ext[0],/silent)
    spfits_spdata2=dinfo2.counts
  
    ;spfits_livet_modu = din2.livet_modu  ;yang
    spfits_livet_spec = dinfo2.livetime  ;yang
    
    ;64->80 energy bands, 221207,230309, yang su, 
    ind_s = [0,10,35,55]   
    ind_e = [9,34,54, nenergy-1]
    
    dsize=size(spfits_spdata1)
    spfits_lcdata1=make_array(4,dsize[2],dsize[3])
    tmp1=total(spfits_spdata1[ind_s[0]:ind_e[0], *,*],1)
    tmp2=total(spfits_spdata1[ind_s[1]:ind_e[1], *,*],1)
    tmp3=total(spfits_spdata1[ind_s[2]:ind_e[2], *,*],1)
    tmp4=total(spfits_spdata1[ind_s[3]:ind_e[3], *,*],1)
    spfits_lcdata1=[[[tmp1]],[[tmp2]],[[tmp3]],[[tmp4]]]
    spfits_lcdata1=transpose(spfits_lcdata1,[2,0,1])
    ;help,spfits_lcdata1
    dsize=size(spfits_spdata2)
    spfits_lcdata2=make_array(4,dsize[2],dsize[3])
    tmp1=total(spfits_spdata2[ind_s[0]:ind_e[0], *,*],1)
    tmp2=total(spfits_spdata2[ind_s[1]:ind_e[1], *,*],1)
    tmp3=total(spfits_spdata2[ind_s[2]:ind_e[2], *,*],1)
    tmp4=total(spfits_spdata2[ind_s[3]:ind_e[3], *,*],1)
    spfits_lcdata2=[[[tmp1]],[[tmp2]],[[tmp3]],[[tmp4]]]
    spfits_lcdata2=transpose(spfits_lcdata2,[2,0,1])
    ;help,spfits_lcdata2
    
    spfits_lcenebins=make_array(4,2,/float)
    spfits_lcenebins[0,0]=spfits_spenebins[ ind_s[0],0]
    spfits_lcenebins[0,1]=spfits_spenebins[ ind_e[0],1]
    spfits_lcenebins[1,0]=spfits_spenebins[ ind_s[1],0]
    spfits_lcenebins[1,1]=spfits_spenebins[ ind_e[1],1]
    spfits_lcenebins[2,0]=spfits_spenebins[ ind_s[2],0]
    spfits_lcenebins[2,1]=spfits_spenebins[ ind_e[2],1]
    spfits_lcenebins[3,0]=spfits_spenebins[ ind_s[3],0]
    spfits_lcenebins[3,1]=spfits_spenebins[ ind_e[3],1]
   
    spstr={spfits_times:spfits_times, spfits_expt:spfits_expt, $
      spfits_livet_spec:spfits_livet_spec,spfits_spenebins:spfits_spenebins, $
      spfits_spdata1:spfits_spdata1, spfits_spdata2:spfits_spdata2, $
      spfits_lcdata1:spfits_lcdata1, spfits_lcdata2:spfits_lcdata2, $
      spfits_lcenebins:spfits_lcenebins}
    return, spstr
    end
  endcase

end