;Kepler equations used to calculate earth's orbit
Function kepler, theta
  ;constants
  T_period = 3.15581184d7   ; Sidereal period of earth in seconds
  e = 0.0163512d
  C = sqrt((1-e)/(1+e))

  theta = double(theta)
  n = fix(theta/(!pi))
  theta = (theta mod !pi)
  EE = 2d*atan(C*tan(0.5d*theta))
  M = EE-e*sin(EE)
  return, 0.5d*T_period*M/(!pi)+n*0.5d*T_period
end

;+
; Mission     : ASO-S
; Name        : sun_north_angle
; Purpose     : Calculate solar rotational obliquity angle to ecliptic in earth view, through Kepler equations.
; Syntax      : Result = sun_north_angle('2022-10-09T07:43:00')
;               Result = sun_north_angle(time_string, nyear=20)
; Inputs      : 
;         time_in - UT time (UT string or seconds, scalar or array)
;                   calculable from 2022-07-06 01:12:41 to NYEAR later.
; Outputs     : 
;         Solar rotational obliquity angle(s) to ecliptic in earth view,
;         positive value(s) for clockwise from the ecliptic north in earth view.
;         (输出太阳自转轴与黄道法线夹角在地球处的投影角度，输出的角度从黄道北顺时针方向为正值)
; Keywords    : 
;         nyear - the calculable year range from basetime (2023-01-04T16:17:00),
;                 10 years for default.
; History     : 
;     2022/08/30, Zhentong Li, written
;     2022/10/14, Zhentong Li, add keyword NYEAR
;     2023/01/09, Zhentong Li, bug fixed.
; Contact     : ztli@pmo.ac.cn
;-
Function sun_north_angle, time_in, nyear=nyear

  beta=7.25d*!dtor   
  ;Solar rotational obliquity to ecliptic.
  ;ref. https://nssdc.gsfc.nasa.gov/planetary/factsheet/sunfact.html
  
  prec = 100  ;the calculated orbital degrees interval is 1/prec.
  basetime = '2023-01-04T16:17:00'  ;Perihelion time as basetime (t=0)
  
  default, nyear, 10.  ;calculable year range
  
  nyear = float(nyear)
  time_sec = anytim(time_in)-anytim(basetime)
  str = atime(anytim(basetime)+31536000.*nyear)+'. '
  if max(time_sec) gt 31536000.*nyear then message, 'The calculable time range is from 06-Jul-22 01:12:41 to '+str+'Please extand the calculable year range using keyword NYEAR.'
  ndegs = (360.*nyear+180.)*prec
  degs0 = 1d/prec*(findgen(ndegs)-180.*prec)*!dtor  ;180的意思是往前计算半个轨道，以便覆盖到发射日期
  degs=degs0
  times = kepler(degs)
  t0 = anytim('2023-03-06T22:32:43')-anytim('2023-01-04T16:17:00')

  theta0 = interpol(degs0, times, t0)
  alpha = atan(tan(beta)*sin(degs0-theta0))*!radeg

  a_out = dblarr(n_elements(time_sec))
  for i=0,n_elements(time_sec)-1 do a_out[i] = interpol(alpha, times, time_sec[i])
  if n_elements(time_sec) eq 1 then a_out = a_out[0]
  return, a_out
end