;+
; Mission     : ASO-S/HXI
; Name        : hxi_get_lightcurve
; Purpose     : get HXI light curve for given Trange and Enrange, and minimum exposure time from Level 1 data
; Category    : data processing; light curves
; Example     :
;   enrange =[25.,50.]
;   trange=  ['2023/01/06 00:54:00','2023/01/06 00:58:00'] 
;   dur_min = 4.  ;4s
;   det_ids = [91,92,93]
;   hxi_get_lightcurve,  trange=trange, enrange=enrange, dur_min=dur_min, det_ids=det_ids,  $
;                        lc_str = lc_str , tcalib=tcalib,tbin_center=tbin_center
; Inputs      :
;       filein      - input filename of level 1 FITS
;       earray      - input energy array of images
;       trange      - time range in double
;       dur_min     - minimum exposure time for time rebinning (seconds, max 4s)
;       det_ids     - selected det_ids; for example, [91] means D91 or detector index 90, [91,92,93] means sum of D91+D92+D93
; Outputs     :
;       lc_str      - output structure, including
    ;       tarray      - output time array
    ;       expt        - exposure time
    ;       cts         - count rate of selected detectors in given erange
    ;       cts_bkg     - count rate of corresponding BKG detectors in given erange
    ;       cts_tot     - count rate of all detectors in given erange
    ;       livet       - livetime
    ;       err, err_bkg,err_tot  - errors
    ;       det_ids     - detector ids of selected detectors (in the range D1-D99)
    ;       det_ids_bkg - detector ids of corresponding bkg detectors (in the range D1-D99)
    ;       unit        - unit of output data
; Keywords:
;     [tcalib]      - if 1, apply time correction to HXI time array.
;     [tbin_center] - if 1, then the output time array is the center of each time interval. 
; History     :
;       2023/Mar/08, Yang Su, first beta version.
;       2023/Dec./31, Yang Su, new version with major changes on many aspects.
; Contact     : yang.su@pmo.ac.cn
;-

pro hxi_get_lightcurve, trange=trange, enrange=enrange, dur_min=dur_min, det_ids=det_ids,  $
                        lc_str = lc_str , tcalib=tcalib,tbin_center=tbin_center

;  delim = path_sep()
;  thisyear = '2022'
;  thismon = '11'
;  thisday = '11'
;
;  files_lev10 = file_search(getenv('hxi_db_lev10')+delim+thisyear+delim+thismon+delim+thisday,'hxi_det_lev10*.fits',FOLD_CASE=0)
;  filein = files_lev10[1]
  
  default, det_ids, [92,93,94]
  det_index = det_ids -1
  
  default, dur_min, 4.
  default, tbin_center, 1
 
  delim=path_sep()
  checkvar, tcalib, 1 ;231230,

  ;yang, 221002, add ; 230531;
  if keyword_set(prod) then dir_lev1 = getenv('hxi_db_lev10') else dir_lev1 = hxi_data_paths() ;getenv('hxi_data_user') ;getenv('HXI_DATA_ARCHIVE') ;230519, YS
 
  tarray_input = trange ; ['2022-09-20T09:22:35.330', '2022-09-20T09:23:45.330'] 
  earray_input = enrange ; [20.,30.] 
 
  this_date = UTC2YYMMDD( anytim(tarray_input[0],/UTC_EXT),/yyyy)
  temp_t= atime(tarray_input[0], /time)
  hr= STRMID(temp_t,strlen(temp_t)-18,2)
 
  tarray_st = atime(anytim(tarray_input[0]) )
  tarray_et = atime(anytim(tarray_input[-1]) )
  
  hxi_find_data, tarray_st, tarray_et,fileout=all_files_lev1, lev10=1 
    
    din0 = mrdfits( all_files_lev1[0], 0,header0,/silent)
    str0=fitshead2struct(header0)

    Date_obs=str0.Date_obs
    Date_end=str0.Date_end
    Date_ref=str0.Date_ref
    tstart=str0.tstart
  
    n_files = n_elements(all_files_lev1)
    t_corr = fltarr(n_files)
    
    for i=0,n_files-1 do begin
 
      this_time = file2time(all_files_lev1[i])
      if keyword_set(tcalib) then t_corr[i] =  hxi_timelag_calib(this_time)/1000.
      
      if i eq 0 then begin
        din1_thisi = mrdfits( all_files_lev1[i], 1,header1,/silent)
        h_time = din1_thisi.time + anytim( Date_ref )
        h_time +=  t_corr[i] 
      endif else begin
        din1_thisi = mrdfits( all_files_lev1[i], 1,header1,/silent)
        this_time = din1_thisi.time + anytim( Date_ref )
        this_time +=  t_corr[i]
        h_time = [h_time, this_time]
      endelse
 
      if i eq 0 then begin
        din0 = mrdfits( all_files_lev1[i], 0,header0,/silent)
        din1 = mrdfits( all_files_lev1[i], 1,header1,/silent)
        din3 = mrdfits( all_files_lev1[i], 3,header3,/silent) ;removed din2, 230318, yang
        din4 = mrdfits( all_files_lev1[i], 4,header4,/silent)
        din7 = mrdfits( all_files_lev1[i], 7,header7,/silent)

      endif else begin
        ; din0 = merge_struct( din0, mrdfits( all_files_lev1[i], 0,/silent)) ;ccx 1124
        din1 = merge_struct( din1, mrdfits( all_files_lev1[i], 1,/silent))
        din3 = merge_struct( din3, mrdfits( all_files_lev1[i], 3,/silent))
        din4 = merge_struct( din4, mrdfits( all_files_lev1[i], 4,/silent))
        din7 = merge_struct( din7, mrdfits( all_files_lev1[i], 7,/silent))
      endelse
 
    endfor
 
  din2 = mrdfits( all_files_lev1[0], 2,header2,/silent) ;din2, 230318, yang
  ;   Primary header: 105 records
  ;   No data
  ;   Extension 1 -- INFO
  ;   Extension 2 -- EBOUNDS
  ;   Extension 3 -- HXI_MODU
  ;   Extension 4 -- HXI_SPEC
  ;   Extension 5 -- HXI_MODU_4CHAN
  ;   Extension 6 -- HXI_SPEC_4CHAN
 
 
  hexpt= din1.exptime
  n_time=n_elements(hexpt)
  ;h_time = atime( din1.time + anytim( Date_ref ) )
 
 
  h_en_l = din2.en_min
  h_en_h = din2.en_max
  
  hcounts_modu =  din3.counts
  hcounts_spec =  din4.counts      ;HXI_SPEC
  
  hlivet_modu =  din3.livetime
  hlivet_spec =  din4.livetime
  
  hcounts   = [[hcounts_modu], [hcounts_spec]]
  hlivet    = [hlivet_modu, hlivet_spec]
  
  ;enrange = [20,70]
  en_min = enrange[0]
  en_max = enrange[-1]
  en_min_index = (where( h_en_l ge en_min))[0]
  en_max_index = (where( h_en_h le en_max))[-1]
 
  det_index_bkg = [94,95,98]
  if array_equal(det_index,91) then det_index_bkg = [94]
  if array_equal(det_index,92) then det_index_bkg = [95]
  if array_equal(det_index,93) then det_index_bkg = [98]
  if array_equal(det_index,[91,92]) then det_index_bkg = [94,95]
  if array_equal(det_index,[91,93]) then det_index_bkg = [94,98]
  if array_equal(det_index,[92,93]) then det_index_bkg = [95,98]
  if array_equal(det_index,[91,92,93]) then det_index_bkg = [94,95,98]
 
  if n_elements(det_index) eq 1 and min(det_index) ge 90 then begin
    if en_min_index eq en_max_index then begin
      counts =  reform( hcounts[en_min_index,det_index,*])
      counts_bkg =  reform( hcounts[en_min_index,det_index_bkg,*])
      counts_tot  = reform( hcounts[en_min_index:en_max_index,*,*]) 
      livet = reform(hlivet[det_index,*])
    endif else begin
      counts =  total(reform( hcounts[en_min_index:en_max_index,det_index,*]) ,1)
      counts_bkg =  total(reform( hcounts[en_min_index:en_max_index,det_index_bkg,*]) ,1)
      counts_tot  =  total( total(reform( hcounts[en_min_index:en_max_index,*,*]) ,1) ,1 )
      livet = reform(hlivet[det_index,*])
    endelse

  endif else begin
    if en_min_index eq en_max_index then begin
      counts =  total(reform( hcounts[en_min_index,det_index,*]) ,1) 
      counts_bkg =  total(reform( hcounts[en_min_index,det_index_bkg,*]) ,1) 
      counts_tot  =  total(reform( hcounts[en_min_index,*,*]) ,1) 
      livet = average( reform(hlivet[det_index,*]),1)
    endif else begin
      counts =  total( total(reform( hcounts[en_min_index:en_max_index,det_index,*]) ,1),1 )
      counts_bkg =  total( total(reform( hcounts[en_min_index:en_max_index,det_index_bkg,*]) ,1),1 )
      counts_tot  =  total( total(reform( hcounts[en_min_index:en_max_index,*,*]) ,1) ,1 )
      livet = average( reform(hlivet[det_index,*]), 1)
    endelse

  endelse
 
    minexpt = dur_min
 
    hxi_rebindata, data1_in=counts, data2_in=counts_bkg, tarray_in = h_time, expt_in=hexpt,data1_out=counts_new, data2_out=counts_bkg_new,$
      expt_out=expt_out, tarray_out = tarray_out, avg_data =0,  minexpt = minexpt, tbin_center = 0

    hxi_rebindata, data1_in=counts_tot, data2_in=livet,expt_in=hexpt,data1_out=counts_tot_new , data2_out=livet_new, $
        avg_data =0,  minexpt = minexpt

    counts=counts_new
    counts_bkg = counts_bkg_new
    counts_tot  =  counts_tot_new
    livet  = livet_new
    exptime = expt_out
    if tbin_center eq 0 then tarray = tarray_out+exptime/2. else tarray = tarray_out
 
   cts = f_div( counts, livet)
   cts_bkg = f_div( counts_bkg, livet)
   cts_tot = f_div( counts_tot, livet)
 
   err = f_div( sqrt(counts) ,livet) 
   err_bkg = f_div( sqrt(counts_bkg) ,livet) 
   err_tot = f_div( sqrt(counts_tot) ,livet)  
 
    ;trange = ['2023-03-07T15:40:00.000', '2023-03-07T15:47:00.000']
 
    timer_l = anytim(tarray_st)
    timer_u = anytim(tarray_et)
    tindex_l = (where(tarray le timer_l ))[-1] 
    tindex_h = (where(tarray ge timer_u ))[0]
 
    cts = cts[tindex_l:tindex_h]
    cts_bkg = cts_bkg[tindex_l:tindex_h]
    cts_tot = cts_tot[tindex_l:tindex_h]
    livet = livet[tindex_l:tindex_h]
    err = err[tindex_l:tindex_h]
    err_bkg = err_bkg[tindex_l:tindex_h]
    err_tot = err_tot[tindex_l:tindex_h]
 
    tarray = tarray[tindex_l:tindex_h]
    expt = exptime[tindex_l:tindex_h]
    stop
    unit = 'counts per second'
    det_ids_bkg = det_index_bkg +1
    lc_str = {tarray: tarray, $
              expt: expt, $
              cts: cts, $
              cts_bkg: cts_bkg, $
              cts_tot: cts_tot, $
              livet: livet, $
              err: err, $
              err_bkg: err_bkg, $
              err_tot: err_tot, $
              det_ids: det_ids, $
              det_ids_bkg:det_ids_bkg,$ 
              unit: unit }
end