;+
; Mission   : ASO-S/HXI
; PURPOSE   : Define hxi_image Object
; CONTRUCTION:
;  o = Obj_New('hxi_image') or  o = hxi_image()
; HISTORY:
;  Writen by Fanxiaoyu Xia, 2021/11/30
;  2022/07/16, Fanxiaoyu Xia, added keyword filename for hxi_image::getdata.
;  2022/08/09, Fanxiaoyu Xia, if filename is empty or wrong, return an error message.
;  2022/08/19, Fanxiaoyu Xia, added pointer image_str and procedure hxi_image::setdata, so that 
;    images and parameters can be saved and output.
;  2022/09/16, Fu Yu, added keyword description.
;  2022/09/16  Fu Yu, added pointer img_input_paras_old, if not change the paams, then not make 
;    image again.
;  2022/09/18, Fanxiaoyu Xia, add keyword data_str, if input standard map structures, plot 
;    images derectly.
;  2022/09/21, Fanxiaoyu Xia, if input imaging parameters or detectors are undefined, return an
;    error.
;  2023/03/31, Fu Yu, re-define the title of plotman with 'plot_args'; add .pro 'hxi_en2str' ,
;    'hxi_draw_info' and 'hxi_draw_info_on_plot' for auxiliary showing some information.
;      /04/07, adjust the way to show cube data with open from .fits file, need simplify latter            
;  2023/04/09, Yang Su, removed 'enbins:' in the title
;  2023/09/20, Changxue Chen, add several keywords for selecting background data (used in hxi_get_image.pro)
;  2024/02/04, Yang Su, add detid_mm for showing detector range on plot.

pro hxi_image::set, time_range=time_range, time_interval=time_interval, $
  energy_bins=energy_bins, collnum=collnum, data_units=data_units, $
  _extra=_extra

  if keyword_set(time_range) then begin
    self.tstart=anytim(time_range[0])
    self.tend=anytim(time_range[1])
  endif

  if keyword_set(collnum) then begin
    self.collnum=ptr_new(collnum)
  endif

  if keyword_set(energy_bins) then begin
    self.energy_bins=ptr_new(energy_bins)
  endif
end

function hxi_image::getdata, tarray=tarray, enebins=enebins, collindex=collindex, det_id =det_id, det_range=det_range, $
              method=method, xc=xc, yc=yc,nxy=nxy, psize=psize, img_calib=img_calib, filename=filename, $
              bkg_obj=bkg_obj,bkgtype=bkgtype,fitmode=fitmode,fit_trange=fit_trange,bkgmode = bkgmode

  n_ints=n_elements(tarray[0,*])
  stime=atime(tarray[0,0])
  etime=atime(tarray[1,n_ints-1])
  file_gap= gt_hour(etime)- gt_hour(stime)
  
  if file_gap eq 0 then begin
    file_nm = strmid( time2file(anytim(stime)),0,11)
    if strmatch(filename, '*hxi_*lev10_'+file_nm+'*.fits', /fold_case) eq 1 then begin
      hxi_get_image,tarray,enebins,subc_group=collindex, det_range=det_range, method=method, det_id =det_id,$ 
        xc=xc, yc=yc,nxy=nxy, psize=psize,nocalib=1-img_calib, hxi_maps=hxi_maps,filename=filename,$
        bkg_obj=bkg_obj,bkgtype=bkgtype,fitmode=fitmode,fit_trange=fit_trange,bkgmode = bkgmode
        return,hxi_maps
    endif
    hxi_find_data,stime,etime,fileout=filename,/lev10
    nofiles=0
    if filename eq '' then begin
      nofiles=1
    endif
    if nofiles eq 1 then return,-1
    
    hxi_get_image,tarray,enebins,subc_group=collindex,$
    det_range=det_range, method=method, det_id =det_id,  xc=xc, yc=yc,nxy=nxy, psize=psize,nocalib=1-img_calib,$
      hxi_maps=hxi_maps,bkg_obj=bkg_obj,bkgtype=bkgtype,fitmode=fitmode,fit_trange=fit_trange,bkgmode = bkgmode
    return,hxi_maps
  endif else begin    
  hxi_find_data,stime,etime,fileout=filename,/lev10
  nfiles=n_elements(filename)
  nofiles=0
  for i=0,nfiles-1 do begin
    if filename[i] eq '' then begin
      nofiles=1
    endif
  endfor
  if nofiles eq 1 then return,-1
  
  hxi_get_image,tarray,enebins,subc_group=collindex, det_range=det_range, method=method, det_id =det_id, $
    xc=xc, yc=yc,nxy=nxy, psize=psize,nocalib=1-img_calib, hxi_maps=hxi_maps, $
    bkg_obj=bkg_obj,bkgtype=bkgtype,fitmode=fitmode,fit_trange=fit_trange,bkgmode = bkgmode
  return,hxi_maps
  endelse
    
end

pro hxi_image::plotman,tarray=tarray, enebins=enebins, collindex=collindex,det_id =det_id, det_range=det_range,$
              method=method, xc=xc, yc=yc,nxy=nxy, psize=psize, img_calib=img_calib, plotman_obj=plotman_obj, filename=filename,$
              data_str=data_str, map_info=map_info, $
               bkg_obj=bkg_obj,bkgtype=bkgtype,fitmode=fitmode,fit_trange=fit_trange,bkgmode = bkgmode  ;img_calib,231222

  IF keyword_set(data_str) then begin
    dim=(size(data_str))[1]
    tmp_plot_args={}
    
    if data_str[0].id eq 'ASO-S HXI X-ray' then begin
      tarray=map_info.time_intervals      ;[2, ntim] for dim > 1; [2] 
      enebins=map_info.energy_band        ;[2, nene] for dim > 1; [2]
      method=map_info.image_algorithm   
      counts_total=map_info.total_counts  ;[nene,ntim]
      counts_bg=map_info.bg_counts 
      
      ntim=n_elements(tarray.dim) gt 1 ? (tarray.dim)[1] : 1
      nene=n_elements(enebins.dim) gt 1 ? (enebins.dim)[1] : 1
      if TAG_EXIST(map_info, detid_mm) eq 0 then detid_mm = intarr(2,nene,ntim)-1 else detid_mm = map_info.detid_mm  ; 240204,YS 
      
      nn=0
      for i=0, ntim-1 do begin
        for j=0,nene-1 do begin
  
          tmp_title = 'ASO-S HXI X-ray | '+ hxi_en2str(enebins[0:1,j]) + ' | '+method+' image!C' + hxi_en2str(tarray[0:1,i],/tim)
          tmp_plot_args={plot_args:{title:tmp_title,ytitle:'Y (arcsec)'}}
          description =hxi_en2str(tarray[0:1,i],/tim)+ ' | '+ hxi_en2str(enebins[0:1,j]) +' | '+method+' img'
          
          plotman_obj -> new_panel,input=data_str[nn], plot_type='image', colortable=5,titles=' ',ytitles=' ', $
            description= description, _extra=tmp_plot_args
            hxi_draw_info, plotman_obj, counts_total=counts_total[j,i],  counts_bg=counts_bg[j,i], detid_mm= reform( detid_mm[*,j,i] )
          nn=nn+1
          
        endfor
      endfor
    endif else begin ;for other input file
      for i=0, dim-1 do begin
        description= data_str[i].id + ' | '+  data_str[i].TIME
        plotman_obj -> new_panel,input=data_str[i], plot_type='image', colortable=5,titles=' ',ytitles=' ', $
          description= description;, _extra=tmp_plot_args
      endfor
    endelse
    
    self.image_str=ptr_new(0)
    return
  ENDIF

  if not keyword_set(det_id) then begin 
    if  det_range eq [-1] then begin
      a = dialog_message('Detectors are undefined!')
      return
    endif
  endif

  if xc eq -10000 or  xc eq -10000 or nxy eq -10000 or psize eq -10000. then begin
    a = dialog_message('Image parameter is undefined!')
    return
  endif

  img_para_change_status=1 
  tmp_paras_new = {tarray:tarray, enebins:enebins, det_range:det_range, method:method,  xc:xc, yc:yc,nxy:nxy, img_calib:img_calib,psize:psize, det_id:det_id $
    ,bkgtype:bkgtype,fitmode:fitmode,fit_trange:fit_trange,bkgmode : bkgmode}

  if not ptr_valid(self.img_input_paras_old) then begin
    self.img_input_paras_old = ptr_new(/allocate)
    *(self.img_input_paras_old) = tmp_paras_new
  endif else begin 
    tmp_paras_old = *(self.img_input_paras_old)
    *(self.img_input_paras_old)=tmp_paras_new    
    for itag=0,n_tags(tmp_paras_old)-1 do begin  ; comparsion for each new input para
      tmp_size_new=size(tmp_paras_new.(itag)) & tmp_size_old=size(tmp_paras_old.(itag))    ;①judge by size
      if n_elements(tmp_size_new) eq n_elements(tmp_size_old) and total((tmp_size_new eq tmp_size_old) eq 0.0) eq 0.0 then begin
        tmp_compare_res = tmp_paras_new.(itag) eq tmp_paras_old.(itag)                     ;②judge by content
        if total(tmp_compare_res eq 0.0) ne 0.0 then img_para_change_status=2.0
      endif else begin
        img_para_change_status=2.0  
      endelse        
      if img_para_change_status gt 1 then break
    endfor
    if img_para_change_status eq 1 then img_para_change_status=0
  
  endelse

  if img_para_change_status ge 1 then begin
    print,'Make new image, according to updated parameters..................'
    box_message,'Imaging starts, please wait...' 
    maps_str=self->getdata(tarray=tarray, enebins=enebins, collindex=collindex,det_id=det_id, $
      det_range=det_range, method=method, xc=xc, yc=yc,nxy=nxy, psize=psize,img_calib=img_calib, filename=filename, $
      bkg_obj=bkg_obj,bkgtype=bkgtype,fitmode=fitmode,fit_trange=fit_trange,bkgmode = bkgmode)
  endif else begin
    print,'No parameter is changed. Imaging stopped!'
  endelse

  if typename(maps_str) eq 'INT' then begin  
    if maps_str eq 0 then begin
      return
    endif else begin
      a = dialog_message('No plotman data found.')
      return
    endelse
  endif
  
  ndim = size(maps_str)
  if ndim[0] eq 1 then begin
    n_times = 1 
    n_ebands = ndim[1] 
  endif else begin
    n_times= ndim[2]
    n_ebands = ndim[1] 
  endelse

  if not keyword_set(plotman_obj) then begin
    plotman_obj=obj_new('plotman')
    return
  endif

  for i=0, n_times-1 do begin
    for j=0,n_ebands-1 do begin
      des_tmp = hxi_en2str(tarray[0:1,i],/tim)+ ' | '+ hxi_en2str(enebins[0:1,j]) +' | '+method+' img'
      tmp_title = 'ASO-S HXI X-ray | '+ hxi_en2str(enebins[0:1,j]) +' | '+method+' image!C' + hxi_en2str(tarray[0:1,i],/tim)
      tmp_plot_args={plot_args:{title:tmp_title,ytitle:'Y (arcsec)'}}
      plotman_obj -> new_panel,input=maps_str[j,i], plot_type='image', colortable=5,titles='',ytitles='', $
        description= des_tmp, _extra=tmp_plot_args
      detid_mm = [reform( min(det_id) ), reform( max(det_id) )]+1 ;240204, YS
      hxi_draw_info, plotman_obj, /counts_tmp,detid_mm=detid_mm
     endfor
  endfor
  self->setdata,data_str=maps_str
  box_message,'Imaging is done!'  
end

pro hxi_image::setdata,data_str=data_str
  self.image_str=ptr_new(data_str)
end

pro hxi_image__define
    self = {hxi_image, $
      tstart:0.d0, tend:0.d0, time_interval:0., collnum:ptr_new(), energy_bins:ptr_new(), data_units:0,$
      image_str:ptr_new(0), img_input_paras_old:ptr_new(0)}
end
