;+
; Mission     : ASO-S/HXI
; Name        : hxi_backproj
; Purpose     : make BP dirty image from HXI data
; Category    : imaging
; Syntax      : 
; Inputs      :
;       counts      - input data in counts, [91,ntime ]
;       exptime     - expousre time in s, [ntime]
;       patt        - pattern, [nxy,nxy,91]
;       totf        - total flux
;       psize/xc/yc/nxy - map parameters: pixel size/xcenter/ycenter/number of pixels along x(y)
;       det_id      - detector/subcollimator index mask
;       [uniformw]  - weighting of subcollimators
;       [no_pot_corr] - if set, pointing correction is not applied for imaging;
;
; Outputs     :
;       bp_image   - reconstructed BP image
;
; Outputs (OPTIONAL):
; History     :
;       Aug. 2019, Yang Su, first beta version, developed for imaging simultion; 
;               are now added; image correction for pointing is added. Yang Su and Wei Chen
;       - Jun. 2022, v1.0 beta, Yang Su, many changes to the code following the development of simulated data;
;       Aug. 2022, Yang Su, added corrections for pointing changes;
;       Sep. 2022, v1.1 beta, Yang Su, a few improvements, including comments and program header; 
;               added no_pot_corr;
;       Oct. 2022 - Apr. 2023, many changes are made to the code based on imaging test of real data.
; Contact     : yang.su@pmo.ac.cn

pro hxi_backproj, counts, exptime,  patt, totf=totf, psize=psize,xc=xc, yc=yc,nxy=nxy, $
                  det_id=det_id, no_pot_corr=no_pot_corr,uniformw= uniformw, bp_image=bp_image

  if not keyword_set(psize) then psize=1. 
  if not keyword_set(exptime) then exptime=1.
  if not keyword_set(totf) then totflux= 1. else totflux=totf/exptime  ;240109,YS
  if not keyword_set(xc) then xc=0  
  if not keyword_set(yc) then yc=0  
  if not keyword_set(nxy) then nxy=129 
  if not keyword_set(det_id) then det_id= indgen(91)
  if not keyword_set(uniformw) then uniformw =0
  if not keyword_set(no_pot_corr) then no_pot_corr =0
  
  if keyword_set(no_pot_corr) then begin
    ;check the size/dim of counts
    datasize = size(counts)
    pattsize = size(patt)
    if datasize[0] eq 1 then counts=reform(counts,datasize[1],1 )
    if datasize[0] eq 2 then counts=reform(total(counts,2),datasize[1],1 )
    if pattsize[0] eq 3 then patt=reform(patt,pattsize[1],pattsize[2],pattsize[3],1 )
    if pattsize[0] ge 4 then begin
       patt=reform( (patt[*,*,*,0] ),pattsize[1],pattsize[2],pattsize[3],1 )
      message, 'Warning! You select [no_pot_corr], pattern is averaged over time.',/con
    endif
    
    ;if keyword_set(totf) then totflux=totf/exptime   ;yang, 221109;240109
  endif
  
  n_det = n_elements(det_id)

  default, uniformw, 0
 
;  angs = (!pi/180.)*ds.angles
;  indc = ds.c_index;con index
;  inds = ds.s_index;sin index
;  pitchs = ds.pitchs
;  ;  D =!pi/180./60./60.
; 
 
  ;spatial_frequency_weight = nsfw eq 1 ? spatial_frequency_weight + fltarr( nvis ): spatial_frequency_weight
 
  if keyword_set( uniformw ) then begin
    ds=hxi_get_config_data()

    angs = (!pi/180.)*ds.grid_angles

    indc = ds.grid_c_index;con index
    inds = ds.grid_s_index;sin index
    pitches = ds.grid_p *0.0001

    DtL = (ds.subcoll_L*!pi/180./60./60.)

    u = DtL* cos( angs[indc])/pitches[indc]
    v = DtL* sin( angs[inds])/pitches[inds]

    sf_weight =sqrt(u^2.+v^2.)   ;uniform weight

    sf_weight =sf_weight/total(sf_weight)
    sf_weight = [rebin(sf_weight, n_elements(sf_weight)*2), sf_weight[-1]]
  endif else begin
    sf_weight = fltarr(91)+1.
  endelse
 
  if (size(reform(counts)))[0] eq 1 then begin
    ntimes =1 
    this_patt= reform( reform(patt[*,*,det_id,0]), (size(patt))[1],(size(patt))[2],n_det*ntimes)
  endif else begin
    ntimes= (size(reform(counts)))[2]
    this_patt= reform( reform(patt[*,*,det_id,*]), (size(patt))[1],(size(patt))[2],n_det*ntimes)
  endelse
  counts = reform(counts,  (size(counts))[1],ntimes)
    
    the_counts = reform( counts[det_id,*], n_det*ntimes )
    the_sf_weight = reform( rebin( sf_weight[det_id],n_det, ntimes),  n_det*ntimes )
    ;spatial_frequency_weight
 
  nx=(size(this_patt))[1]
  ny=(size(this_patt))[2]
  n_det_new= (size(this_patt))[3]  ;n_det*n_timeint
  
  newd=fltarr(nx, ny, n_det_new)
  
  p_avg= average(this_patt,3)
  p_avg2=  total((this_patt - rebin(p_avg,nxy,nxy, n_det_new, /sample))^2. ,3)/float(n_det_new-1)
  for ii= 0, n_det_new-1  do  newd[*,*,ii]=  f_div( the_sf_weight[ii]*the_counts[ii] * ( reform(this_patt[*,*,ii])- p_avg), p_avg2 )     ;/(0.5*22.)

  bb=average( newd,  3)
  bp_image= bb

end
