;+
; Mission     : ASO-S/HXI
; Name        : hxi_bkg_fit
; Purpose     : fit non-flare MODU detector data as background data
; Category    : data processing and production; imaging
; Calls       : fit_backgrnd.pro
; Syntax      :
; Inputs      :
;       fit_trange    - time periods to be fitted
;       erange        - energy ranges to be fitted
;       fitmode       - fitting mode, including 0/1/2 Poly temporarily
;       expts         - array of exposure times in imaging time range (note: not fitting time range)
;
; Outputs     :
;       fit_crate      - fitted count rate in selected time range (fit_range) 
;       fit_counts     - counts in selected time range
;;example     :
;     fit_trange = [['23-feb-23 06:08:27','23-feb-23 06:10:30']]
;     erange = [[20,30],[30,50]]
;     fitmode = '0Poly'
;     hxi_bkg_fit,fit_trange=fit_trange,erange=erange,fitmode=fitmode,expts=expts,fit_crate=fit_crate,fit_counts=fit_counts
;
; History     :
;       2023/09/20, Changxue Chen, the first version
;       2023/11/01, Changxue Chen, add a new fitting method 'BKG Profile', delete '2Poly; fix a bug when using '1Poly'
;       2023/11/04, Changxue Chen, modify the input keyword 'selected' of fit_backgrnd.pro
;       2023/11/09, Changxue Chen, add new output keyword 'bkg_counts4plot' 
;       2023/12/04, Changxue Chen, fix a bug caused by incorrect data dimensions
pro hxi_bkg_fit,fit_trange=fit_trange,erange=erange,fitmode=fitmode,expts=expts,$
    fit_crate=fit_crate,fit_counts=fit_counts,im_trange = im_trange,bkg_counts_rate = bkg_counts_rate,$
    bkg_counts4plot = bkg_counts4plot
 ; if not keyword_set(fitmode) then fitmode=1
  
  if n_elements(size(fit_trange)) eq 5 then begin ;UT format
    n_time = (size(fit_trange))[2]
  endif else begin
    n_time =1
  endelse
  
  if n_elements(size(erange)) eq 5 then begin
    n_eng = (size(erange))[2]
  endif else begin
    n_eng =1
  endelse
  
 ;-extract data in fitting time range--------------------------------------------------
 for i=0,n_time-1 do begin
;   hxi_get_counts4imaging_old,erange, fit_trange[*,i], counts_this, counts_bg_this, tot_counts, tot_counts_bg, cent_xy $
;     ,tarray_out, expt_this
   
   hxi_get_counts4imaging, erange, fit_trange[*,i],  counts_str=counts_str
   
   counts_this = counts_str.counts_subcoll
   counts_bkgdet_center = reform(counts_str.counts_bkgdet[0,*,*])
   counts_bkgdet_edge =counts_str.counts_bkgdet_edge
   tarray_out = counts_str.tarray_out
   expt_this = counts_str.expt
   counts_det= counts_str.counts_totdet
   ;  tot_counts_bg= counts_str.counts_totdet_bkg
   ;livet = counts_str.livet_subcoll
  ;livet_bg = counts_str.livet_bkgdet
   
   tmp_ntime = n_elements(tarray_out)
   counts_rate = make_array(91,tmp_ntime,n_eng)
   ;this_bkg_counts_rate = make_array(91,tmp_ntime,n_eng)
   counts_bkgdet_tot =  make_array(n_eng)
   counts_specdet_tot =  make_array(n_eng)
   counts_specdet = make_array(tmp_ntime,n_eng)
   
   for j=0,n_eng-1 do begin
     for k=0,90 do begin
      counts_rate[k,*,j] = counts_this[k,*,j]/expt_this
     endfor
     
     counts_bkgdet_tot[j] = total((counts_bkgdet_center[*,j]+counts_bkgdet_edge[*,j])/2)
     counts_specdet_tot[j] = total(counts_det[*,j])
     
   endfor
   
   if i eq 0 then begin
     tot_data_rate = counts_rate
     tot_data = counts_this
     tot_time = tarray_out
     tot_expts = expt_this
     tot_bkgdet_data = counts_bkgdet_tot
     tot_specdet_data = counts_specdet_tot
   endif else begin
     tot_data_rate = [[tot_data_rate],[counts_rate]]
     tot_data = [[tot_data],[counts_this]]
     tot_time = [tot_time,tarray_out]
     tot_expts = [tot_expts,expt_this]
     tot_bkgdet_data = [[tot_bkgdet_data],[counts_bkgdet_tot]]
     tot_specdet_data = [[tot_specdet_data],[counts_specdet_tot]]
   endelse

 endfor
 if n_time eq 1 then begin ; ccx 1204
   factor_spec2modu = tot_specdet_data/  total(average(tot_data,1),1)
 endif else begin
   factor_spec2modu = total(tot_specdet_data,2)/  total(average(tot_data,1),1)
 endelse
 
; stop
 case fitmode of
   '0Poly':order=0
   '1Poly':order=1
   ;'2Poly':order=2
   'BKG Profile':order=3
   else: 
 endcase
 
 times = anytim(tot_time)
 ntime = n_elements(times)
 ;sel = [0,ntime-1]
 sel = indgen(ntime)
 bkcounts = make_array(91,ntime,n_eng)
 
 ;stop
 if order lt 2 then begin
   order_in = order
   case order of 
    0:coeff_value = make_array(91,1,n_eng)
    1:coeff_value = make_array(91,2,n_eng)
    else:
   endcase
   message,/info,'Fitting...'
   
   for m=0,90 do begin
     for n =0,n_eng-1 do begin
       bkcounts[m,*,n] = fit_backgrnd(times - min(times), $  ; wants relative times 2xn sec
         tot_data_rate[m,*,n], sqrt(tot_data_rate[m,*,n]), $
         order_in, selected=sel, sigma=sigma ,ltime=tot_expts,coeff_value=para);, exp=do_exp)
       coeff_value[m,*,n] = reform(para)
     endfor
   endfor
   
 endif else begin
  
  if order eq 3 then begin 
    factors = make_array(91,n_eng)
    for j=0,n_eng-1 do begin
      for k=0,90 do begin
        tot_data = tot_data_rate[k,*,j]*tot_expts
        factors[k,j] =  total(tot_data)/  total(tot_bkgdet_data[j,*])
      endfor
    endfor
    
  endif 

 endelse

 ;do interpol/calculation----------------------------
 nexpt = n_elements(expts)
 fit_crate = make_array(91,nexpt,n_eng)
 fit_counts = fit_crate
  bkg_counts4plot = make_array(nexpt,n_eng)
  
 ;dt_start = anytim(tot_time[0])-anytim(fit_trange[0,0])
 start_time = anytim(im_trange[0,0])
 ;fit_times = total(tot_expts,/cumu)-tot_expts/2.+anytim(fit_trange[0,0])
 relative_im_times = total(expts,/cumu)+start_time-anytim(tot_time[0])-expts/2.
  
 for j=0,n_eng-1 do begin
   for k=0,90 do begin
   ; fit_crate[k,*,j] = interpol(bkcounts[k,*,j], fit_times, im_times )
    case order of 
      0:fit_crate[k,*,j] = make_array(nexpt,value =coeff_value[k,*,j] )
      1:fit_crate[k,*,j] =  relative_im_times*coeff_value[k,1,j]+coeff_value[k,0,j]
      3:begin
        ;fact = rebin([coeff_value[k,0,j]/bkg_fit_par[j]], nexpt)
        fit_crate[k,*,j] = factors[k,j]*reform(bkg_counts_rate[*,j])
        end
     else:
    endcase
    
    fit_counts[k,*,j] = fit_crate[k,*,j] * expts
   endfor
   bkg_counts4plot[*,j] = average(fit_counts[*,*,j],1)*factor_spec2modu[j]
 endfor
 
 
end

;times_arr = times - min(times)