;+
; Mission     : ASO-S/HXI
; Name        : hxi_get_counts4imaging
; Purpose     : read, merge corresponding FITS files, and get data for imaging
; Category    : imaging
; Syntax      :
; Inputs      :
;       earray      - input energy ranges [2, nenergy]
;       trange      - input time intervals, [2,1 ]
;       [filename]  - lev1.0 FITS;
; Outputs     :
;       counts_str - output data structure
;           counts_subcoll      - counts data of modulation detectors for given time intervals and energy ranges
;           counts_bkgdet   - background data of BKG detectors at the center for given time interval and energy ranges
;           counts_bkgdet_edge   - background data of BKG detectors at the edges for given time interval and energy ranges
;           counts_totdet   - total counts, [n original time intervals]
;           counts_totdet_bkg   - total bkg counts, [n original time intervals]
;           tarray_out:  
;            expt:  
;            livet_subcoll: 
;            livet_totdet:  
;            livet_bkgdet:  
;            livet_bkgdet_edge:  
;       cent_xy     - locations (in arcsec) of solar disk center, relative to the center of SAS FOV
; Outputs (OPTIONAL):
; History     :
;       Nov. 2021 - Feb. 2022, beta versions, Yang Su, developed for production of levQ2 FITS;
;       Jul. 2022, v1.0 beta, Yang Su, many changes to the code due to the development of simulated data;
;       Aug. 2022, Yang Su, changed the code to get counts in original time intervals instead of counts summed over time;
;               this data is used for pointing correction;
;       2022/08/10, Fanxiaoyu Xia, added keyword filename;
;       2022/09/24, Yang Su, bg_counts -> counts_bg
;       2022/09/27, Yang Su, a few minor changes, program header and comments;
;       2022/11/09, Yang Su, add output for expt.
;       2023/09/01-29, Yang Su, add tarray_out and livetime; changed output to a single structure;
;       2023/11/24, Changxue Chen, can read multi hours data
;       2023/12/30, Yang Su, add keyword tcalib
;       2024/01/18, Yang Su, add cent_indm
; Contact     : yang.su@pmo.ac.cn
;-

pro hxi_get_counts4imaging, earray, trange, counts_str=counts_str, $
                cent_xy=cent_xy, cent_indm=cent_indm, prod=prod, tcalib=tcalib, filename=filename

  delim=path_sep()
  checkvar, tcalib, 1 ;231230,
  
  ;yang, 221002, add ; 230531;
  if keyword_set(prod) then dir_lev1 = getenv('hxi_db_lev10') else dir_lev1 = hxi_data_paths() ;getenv('hxi_data_user') ;getenv('HXI_DATA_ARCHIVE') ;230519, YS
  
  ;hxi_get_lvl1_files, trange, fits_lvl1

  tarray_input = trange ;[['2022-09-20T09:22:35.330', '2022-09-20T09:23:45.330'],['2022-09-20T09:23:46.330', '2022-09-20T09:23:47.330'] ]
  earray_input = earray ;[[20.,30.],[30.,50.],[50.,100.]]

  ;nframes= (size(tarray_input,/dim))[2]

  if (size(tarray_input))[0] eq 1 then tarray_input =  reform(tarray_input, 2,1)
  if (size(earray_input))[0] eq 1 then earray_input =  reform(earray_input, 2,1)

  nenergy= (size(earray_input ))[2]

  this_date = UTC2YYMMDD( anytim(tarray_input[0],/UTC_EXT),/yyyy)
  temp_t= atime(tarray_input[0], /time)
  hr= STRMID(temp_t,strlen(temp_t)-18,2)

  ;221109, yang; 230318, yang
  tarray_st = atime(anytim(tarray_input[0]) )
  tarray_et = atime(anytim(tarray_input[-1]) )
  date_st = atime( tarray_st ,/date)
  date_et = atime( tarray_et ,/date)
  hour_st = STRMID(atime(  anytim(tarray_st), /TIME  ),0,2)
  ; hour_et = STRMID(atime(  anytim(tarray_st), /TIME  ),0,2)
  hour_et = STRMID(atime(  anytim(tarray_et), /TIME  ),0,2) ; ccx 1124

  ; nhours = fix((anytim( date_st+' '+hour_st+':00:00.000')-anytim( date_et+' '+hour_et+':00:00.000'))/60./60.)+1
  nhours = fix((anytim( date_et+' '+hour_et+':00:00.000')-anytim( date_st+' '+hour_st+':00:00.000'))/60./60.)+1 ; ccx1124
  all_dates = UTC2YYMMDD( anytim(anytim(tarray_st)+ findgen(nhours)*3600.,/UTC_EXT),/yyyy)
  all_hours = STRMID(atime(  anytim(tarray_st)+ findgen(nhours)*3600., /TIME  ),0,2)

  if not keyword_set(filename) then begin
    ;find the lev1 file
    all_files_lev1 = make_array(nhours, value=' ')

    for i=0,nhours-1 do begin
      this_year = STRMID(all_dates[i],0,4)
      this_mon = STRMID(all_dates[i],4,2)
      this_day = STRMID(all_dates[i],6,2)

      this_file_lev1= 'hxi_det_lev10_'+all_dates[i] + '_'+all_hours[i]+'*.fits'

      ;yang, 221002, add
      if keyword_set(prod) then begin
        all_files_lev1[i] =dir_lev1+delim+$
          this_year+delim+this_mon+delim+this_day +delim+this_file_lev1
      endif else begin
        temp_files = file_search(dir_lev1 ,this_file_lev1)  ;230531,
        all_files_lev1[i] = hxi_get_recent_file(temp_files[uniq(temp_files)]) ;ccx1124
      endelse
      if file_exist(all_files_lev1[i] ) then begin
        if i eq 0 then begin
          din0 = mrdfits( all_files_lev1[i], 0,header0,/silent)
          din1 = mrdfits( all_files_lev1[i], 1,header1,/silent)
          din3 = mrdfits( all_files_lev1[i], 3,header3,/silent) ;removed din2, 230318, yang
          din4 = mrdfits( all_files_lev1[i], 4,header4,/silent)
          din7 = mrdfits( all_files_lev1[i], 7,header7,/silent)
        endif else begin
          ; din0 = merge_struct( din0, mrdfits( all_files_lev1[i], 0,/silent)) ;ccx 1124
          din1 = merge_struct( din1, mrdfits( all_files_lev1[i], 1,/silent))
          din3 = merge_struct( din3, mrdfits( all_files_lev1[i], 3,/silent))
          din4 = merge_struct( din4, mrdfits( all_files_lev1[i], 4,/silent))
          din7 = merge_struct( din7, mrdfits( all_files_lev1[i], 7,/silent))
        endelse
      endif

    endfor
  endif else begin
    all_files_lev1=filename
    din0 = mrdfits( all_files_lev1, 0,header0,/silent)
    din1 = mrdfits( all_files_lev1, 1,header1,/silent)
    din3 = mrdfits( all_files_lev1, 3,header3,/silent)
    din4 = mrdfits( all_files_lev1, 4,header4,/silent)
    din7 = mrdfits( all_files_lev1, 7,header7,/silent)
  endelse

  din2 = mrdfits( all_files_lev1[0], 2,header2,/silent) ;din2, 230318, yang
  ;   Primary header: 105 records
  ;   No data
  ;   Extension 1 -- INFO
  ;   Extension 2 -- EBOUNDS
  ;   Extension 3 -- HXI_MODU
  ;   Extension 4 -- HXI_SPEC
  ;   Extension 5 -- HXI_MODU_4CHAN
  ;   Extension 6 -- HXI_SPEC_4CHAN

  str0=fitshead2struct(header0)

  Date_obs=str0.Date_obs
  Date_end=str0.Date_end
  Date_ref=str0.Date_ref
  tstart=str0.tstart
  tend=str0.tend

  num_spec=str0.num_spec

  exptime= din1.exptime
  n_time=n_elements(din1)
  time_array= atime( din1.time + anytim( Date_ref ) )
  ;time correction, 2023/12/30,Yang Su 
  if keyword_set(tcalib) then time_array +=  hxi_timelag_calib(time_array)/1000.

  counts_img = din3.counts
  counts_spec = din4.counts

  ;230929,YS, add livetime
  livet_img = din3.livetime
  livet_spec = din4.livetime

  en=  ((din2.en_min)+(din2.en_max))/2.

  find_thist_s = where( anytim(time_array) ge anytim(tarray_input[0,0]) )
  thist_s = find_thist_s[0]
  find_thist_e = where( anytim(time_array)+exptime le anytim(tarray_input[1,0]) )
  thist_e = find_thist_e[-1] > (thist_s+1)

  counts = fltarr( (size(counts_img))[2],thist_e-thist_s+1, nenergy )
  counts_bg = fltarr(  thist_e-thist_s+1, nenergy )
  counts_bg_edge = fltarr(  thist_e-thist_s+1, nenergy )
  tot_counts = fltarr( thist_e-thist_s+1, nenergy )
  tot_counts_bg = fltarr( thist_e-thist_s+1, nenergy )

  for ie=0, nenergy-1 do begin
    find_thise_s = where( anytim(en) ge anytim(earray_input[0,ie]) )
    thise_s = find_thise_s[0]
    find_thise_e = where( anytim(en) le anytim(earray_input[1,ie]) )
    thise_e = find_thise_e[-1]

    counts[*,*,ie]= total( counts_img[thise_s:thise_e, *,thist_s:thist_e ],1)
    counts_bg[*,ie]= average(total( counts_spec[thise_s:thise_e, [3,4,5,6],thist_s:thist_e ],1),1) ;231230,YS, add 7;240109, YS,7->5,6
    counts_bg_edge[*,ie]= average(total( counts_spec[thise_s:thise_e, [5,6,7],thist_s:thist_e ],1),1) ; 230929,YS
    ;tot_counts[*,ie] =  average( total( counts_spec[thise_s:thise_e, 0:1,thist_s:thist_e ],1) ,1)
    ;230905,YS, use D93
    tot_counts[*,ie] =  average( total( counts_spec[thise_s:thise_e, [0,1],thist_s:thist_e ],1),1)
    tot_counts_bg[*,ie] =  average(total( counts_spec[thise_s:thise_e, [3,4],thist_s:thist_e ],1),1) ;240109, YS,

  endfor

  expt =  exptime[thist_s:thist_e]

  ;230929,YS, add livetime
  livet = livet_img[*,thist_s:thist_e]
  ;livet_tot = reform( livet_spec[1,thist_s:thist_e])
  cts_d92= reform( total( counts_spec[thise_s:thise_e, 0,thist_s:thist_e ],1))/reform( livet_spec[0,thist_s:thist_e])
  cts_d93= reform( total( counts_spec[thise_s:thise_e, 1,thist_s:thist_e ],1))/reform( livet_spec[1,thist_s:thist_e])
  cts_d9293_tot = (cts_d92+cts_d93)*expt/2.
  livet_tot = f_div(tot_counts,cts_d9293_tot)*expt

  ;livet_bg = average( livet_spec[[3,4],thist_s:thist_e],1)
  cts_d95= reform( total( counts_spec[thise_s:thise_e, 3,thist_s:thist_e ],1))/reform( livet_spec[3,thist_s:thist_e])
  cts_d96= reform( total( counts_spec[thise_s:thise_e, 4,thist_s:thist_e ],1))/reform( livet_spec[4,thist_s:thist_e])
  cts_d9596_bg = (cts_d95+cts_d96)*expt/2.
  livet_bg = f_div(tot_counts_bg,cts_d9596_bg)*expt

  ;livet_bg_edge = average( livet_spec[[5,6,7],thist_s:thist_e],1)
  cts_d97= reform( total( counts_spec[thise_s:thise_e, 5,thist_s:thist_e ],1))/reform( livet_spec[5,thist_s:thist_e])
  cts_d98= reform( total( counts_spec[thise_s:thise_e, 6,thist_s:thist_e ],1))/reform( livet_spec[6,thist_s:thist_e])
  cts_d99= reform( total( counts_spec[thise_s:thise_e, 7,thist_s:thist_e ],1))/reform( livet_spec[7,thist_s:thist_e])
  cts_edge_bg = (cts_d97+cts_d98+cts_d99)*expt/3.
  livet_bg_edge = f_div(counts_bg_edge,cts_edge_bg)*expt

  sas_t = atime( din7.time + anytim(date_ref) )
  ;time correction, 2023/12/30,Yang Su
  if keyword_set(tcalib) then sas_t +=  hxi_timelag_calib(sas_t)/1000.

  sdc_4c = (din7.SDC_4C)[0:1,*]
  sdc_cen = din7.SDC_CEN ; 240118, YS
  
  ;;230911, FU Yu, test, machine learning result for 2023-Jun-20 flare
  ;ml_sas_replace_fqm_pro, sas_t, sdc_4c

  det_t = atime( anytim(time_array[thist_s:thist_e])+expt/2. )  ;exptime->expt, 230915, CX Chen
  tarray_out = det_t ; 230901, YS

  the_x =   interpol( reform(sdc_4c[0,*]), anytim(sas_t),anytim(det_t) )
  the_y =   2048.5-interpol( reform(sdc_4c[1,*]), anytim(sas_t),anytim(det_t) )
  
  ; 240118, YS
  the_x_cen =   interpol( reform(sdc_cen[0,*]), anytim(sas_t),anytim(det_t) )
  the_y_cen =   2048.5-interpol( reform(sdc_cen[1,*]), anytim(sas_t),anytim(det_t) )
 
  the_sun = get_sun(anytim(time_array[thist_s],/ints), sd=rsun)
  ;psize_sas = rsun/average( (din7.SDC_4C)[2,*])  ;may have problem when rad. is wrong

  psize_sas =0.9367931    ; 0.95156252

  ;221220,[-22,-32.5],  cos( (atan(22./32.5)+(90.-44.1))*!pi/180.)*sqrt(22.^2.+32.5^2.)
  sas_oa = [968.7,991.6]   + ( [40.5,   -37.] )/psize_sas
  sas_oa = [968.7,991.6]   + ( [40.5-3.,   -37. +14.] )/psize_sas

  sas_oa = [1024.,1024.] ;240115,YS, test
 
  cent_xy = transpose( [[the_x-sas_oa[0]], [ the_y -sas_oa[1]]] )  *psize_sas  ; arcsec/pixel  ; 240115,test
  cent_indm = transpose( [[the_x_cen-sas_oa[0]], [ the_y_cen -sas_oa[1]]] )  *psize_sas  ; arcsec/pixel  ; 240118,YS,test

  counts_bg =  transpose( rebin(counts_bg, thist_e-thist_s+1, nenergy,(size(counts))[1], /sample ), [2,0,1])

  ;230929,YS; 231214, YS
  counts_str = {counts_subcoll: counts, $
    counts_bkgdet: counts_bg, $
    counts_bkgdet_edge: counts_bg_edge, $
    counts_totdet: tot_counts, $
    counts_totdet_bkg: tot_counts_bg, $
    tarray_out: tarray_out, $
    expt: expt, $
    livet_subcoll: livet, $
    livet_totdet:  livet_tot, $
    livet_bkgdet: livet_bg, $
    livet_bkgdet_edge: livet_bg_edge}
 
 
end