;+
; Mission     : ASO-S/HXI
; Name        : hxi_get_image
; Purpose     : get reconstructed HXI images from HXI data
; Category    : data processing and production; imaging
; Syntax      : 
; Inputs      :
;       tarray      - input time array of images
;       earray      - input energy array of images
;       det_id/det_range/subc_group - selection of subcollimators
;       det_tot/det_bg  - ?
;       psize/xc/yc/nxy - map parameters: pixel size/xcenter/ycenter/number of pixels along x(y)
;       filename        - filename for the output imagecube FITS
;       method          - imaging method
;       weighting       - weighting of subcollimators
;       niter           - para. for Clean, number of iterations
;       gain_fac        - para. for Clean, gain factor
;       [fulld]         - para. for Clean, FOV of full disk (used for locating flares)
;
; Outputs     :
;       rec_image     - output reconstructed images
;       hxi_maps      - output maps
;
; Outputs (OPTIONAL):
; History     :
;       Jan 2022- June 2022, Yang Su, beta versions for data production based on 
;               simulated data v1.6, v1.7, v1.8,and v2.0; only BP and Clean are used.
;       June 2022 - Aug 2022, Yang Su and Wei Chen, v1.0-v1.1 (beta); Imaging method based on visibilities 
;               are now added; image correction for pointing is added. 
;       2022/08/09  Fanxiaoyu Xia, if keyword method is set to HXI_Backproj or HXI_Clean, return
;                 images to image object.
;       2022/08/09  Fanxiaoyu Xia, added keyword filename.
;       2022/09/19  Fanxiaoyu Xia, Changxue Chen, added vis_bp, vis_clean for keyword method.
;       2022/09/26  Yang Su, Wei Chen, changed the overall software framework for imaging; pointing correction is now working for vis_based methods.
;       2022/10 - 2023/04/09, Yang Su, many changes are made to the code based on observation data.
;       2023/04/18, Yang Su, only consider the selected detectors in getting counts_img; default minexpt is set to 1s. 
;       2023/05/10, Changxue Chen, fixed a bug about invalid setting of keyword det_id
;       2023/09/20, Changxue Chen, added bkg_obj,bkgtype,fit_trange,fitmode,bkgmode keywords to select bkg data
;       2023/10/23, Changxue Chen, adjusted the input order of background data 
;       2023/12/13,2024/02/05, Yang Su, New way of rebinning data; new solar aspect solution; 
; Contact     : yang.su@pmo.ac.cn
;-
   
pro hxi_get_image, tarray,  earray, $
  det_id=det_id, det_range = det_range, subc_group=subc_group, $
  tarray_obs = tarray_obs,$    ;output time array, 230903 
  det_tot=det_tot,det_bg=det_bg, prod=prod, $
  psize=psize, xc=xc, yc=yc, nxy=nxy, nocalib=nocalib, $  ; para for image
  filename=filename, method=method,  weighting = weighting, no_pot_corr=no_pot_corr,  $
  bkgmode = bkgmode ,$  ; 230908, YS, -1:-48h, 0:before and/or after flare? 1: +48h
  bkg_obj=bkg_obj,bkgtype=bkgtype,fitmode=fitmode,fit_trange=fit_trange, $
  niter=niter, gain_fac=gain_fac, fulld=fulld,hxi_fov=hxi_fov, $ ; para for Clean
  rec_image= rec_image, hxi_maps=hxi_maps
 
  default,  loc_x,0.
  default,  loc_y,0.   ;arcsec
  ;default,  psize,1.
  ;default,  fov,200
  default,  ideal, 0
  default,  en_ph, 30.  ; keV
  ;default,  xc,380.
  ;default,  yc,220.
  
  ;default, method ,'HXI_BackProj'
  default, method ,'HXI_Clean'
  default, fulld , 0
  default, prod, 0
  default, hxi_fov, 0
  default, sas_fov, 0
  default, minexpt, 1.
  default, nxy, 151
  default, no_pot_corr,0
  default, maxshift, 0.5 ; 240309, YS: below which pointing correction is turned off (no_pot_corr=1). under test.
  default, bkgmode, -1 ;230908,YS, default : -48h for bkg
  default, nocalib, 1 ;231222, YS
  default, bwf, 2.

  ;det_range/subc_group/FEE are converted to det_id, yang 220924 
  if not keyword_set(det_id) then begin ;cxchen 20230510
    if keyword_set(det_range) or keyword_set(subc_group) then begin
      det_id= hxi_get_detids( det_range=det_range, subc_group=subc_group)
    endif
  endif
 
  checkvar, det_id, indgen(91)
  
  detid_mm = [reform( min(det_id) ), reform( max(det_id) )]+1 ;240204, YS, maxmin detector id
  
  if (nxy mod 2) ne 1 then nxy= nxy+1
  psize = ( float(round(psize)) > 1. ) ;20240209,YS
  fov = nxy*psize
  loc =[xc,yc]
  patt_psize=psize

  if (size(tarray))[0] eq 1 then tarray =  reform(tarray, 2,1)
  if (size(earray))[0] eq 1 then earray =  reform(earray, 2,1)
  
  n_time=   n_elements(tarray[0,*]) ;(size(tarray))[2]
  n_energy = n_elements(earray[0,*])
  n_image =  n_time*n_energy
  
  ds=hxi_get_config_data()

  rec_image = fltarr( nxy, nxy, n_energy, n_time )
  time_title = make_array(n_energy, n_time , value=' ')
  timerange = [atime(anytim(tarray[0,0])-600),atime(anytim(tarray[-1,-1])+600)] ;mod 0916
  timerange = reform(timerange,2,1)
 
  pa = ( sun_north_angle(tarray[0]) )[0] ;+0.7 
  
  theta_sas = 44.54 
  theta1 = ( theta_sas - pa )
  theta2 =  pa
 
  if hxi_fov eq 1 then begin
    pa = 0. 
    theta1 =  theta_sas - pa
    theta2 =  pa
  endif
   
  ;yang, now use make_map for HXI FOV
  hxi_map = make_map(fltarr(nxy, nxy),xc=xc,yc=yc,dx=psize,dy=psize,time=' ',$
                  roll_angle=0,id='ASO-S/HXI ',$
                  err=err,roll_center=[0.,0.],$   ;20221015,yang su
                  _extra=extra,xunits='arcsec',yunits='arcsec',$
                  dur=0., unit='photons cm-2 s-1 asec-2',ALGORITH= ' ' , counts=0.d, bkg =0.d, detid_mm=[1,91], $   ;add detid_mm, 240204,YS
                  energy_l= 0.,energy_h=0., p2c=0.)

  hxi_maps  = replicate(  hxi_map,  n_energy, n_time)
  if n_energy eq 1 or n_time eq 1 then hxi_maps = reform( hxi_maps,n_energy, n_time )
  
  tarray_obs = tarray
  
  for i=0, n_time-1 do begin;1st
    ;get_data
;    hxi_get_counts4imaging_old, earray, tarray[*,i], counts_thisti, counts_bg_thisti, $
;      tot_counts_thisti, tot_counts_bg_thisti, cent_xy,tarray_out,expt, prod=prod, filename=filename  ;for HXI GUI, prod=0, 221015,yang su
;    
    hxi_get_counts4imaging, earray, tarray[*,i],  counts_str=counts_str,cent_xy=cent_xy

    counts_thisti = counts_str.counts_subcoll
   ; counts_bkgdet_center = reform(counts_str.counts_bkgdet[0,*,*])
    ;counts_bkgdet_edge =counts_str.counts_bkgdet_edge
    tot_counts_thisti= counts_str.counts_totdet
    tot_counts_bg_thisti= counts_str.counts_totdet_bkg
    tarray_out = counts_str.tarray_out
    expt = counts_str.expt
    ;livet = counts_str.livet_subcoll
   ; livet_bg = counts_str.livet_bkgdet
    
    ;230903, YS, use observation time rather than input time  
    this_exptime=anytim(tarray_out[-1])-anytim(tarray_out[0])
    this_trange = [tarray_out[0],tarray_out[-1]]
    tarray_obs[*,i] = this_trange
 
    xy_in_sas = cent_xy*0.
        xy_in_sas[0,*] = ( ( xc) *cos(-theta1/180.*!pi) + ( yc)*sin(-theta1/180.*!pi)  )+ cent_xy[0,*]
        xy_in_sas[1,*] = ( ( yc)* cos(-theta1/180.*!pi) - ( xc)* sin(-theta1/180.*!pi) )+ cent_xy[1,*]
 
    shift_sasp_fov =[15.2029 ,-3.10413] ;test for flare locations, based on 230509 flare,b
    ;shift_sasp_fov =[14.6427 ,-4.26349 ] ;90-44.54
    
    cent_xy_FOV = cent_xy*0.
    cent_xy_FOV[0,*] =  (( xy_in_sas[0,*]) *cos(theta_sas/180.*!pi) + (xy_in_sas[1,*])*sin(theta_sas/180.*!pi) )+shift_sasp_fov[0]
    cent_xy_FOV[1,*] =  (( xy_in_sas[1,*])* cos(theta_sas/180.*!pi) - (xy_in_sas[0,*])* sin(theta_sas/180.*!pi) ) +shift_sasp_fov[1]
 
  ;;------------------------------
    bkg_str= bkg_obj.getdata(time_range=tarray[*,i], bkgtype=bkgtype,e_range=earray,bkgmode=bkgmode $
      ,fitmode=fitmode,fit_trange=fit_trange)
 
      for j=0, n_energy-1 do begin ;2st 
 ;        bkg_str= bkg_obj.getdata(time_range=tarray[*,i], bkgtype=bkgtype,e_range=earray[*,j],bkgmode=bkgmode $
 ;         ,fitmode=fitmode,fit_trange=fit_trange) ; ccx 0920
          
          counts = reform( counts_thisti[*,*,j])
          ;counts_bg = reform( counts_bg_thisti[*,*,j])
          counts_bg = reform(bkg_str.bkg_counts[*,*,j])
          tot_counts = reform( tot_counts_thisti[*,j])
          tot_counts_bg = reform( tot_counts_bg_thisti[*,j])
 
          ;yang, 20220928
;          if counts_img lt 5000. then begin
;            a = dialog_message('Warning! there are not enough counts for imaging (<5000). Proceed?', /QUESTION)
;          endif
          
          ;if keyword_set(det_bg) then counts_bg = rebin( average( det_bg,2), (size(counts))[1], (size(counts))[2],/sample)
          
; ----------if need execute following command ?-----------
          ;counts_bg= counts_bkg  *  ( total( counts_bg[0,*],2) / total( counts_bkgdet[0,*],2) )[0]
        
           ;231213, YS, 
        hxi_rebindata, data1_in=counts, data2_in=counts_bg,expt_in=expt,data1_out=counts_new, data2_out=counts_bg_new,$
                expt_out=expt_out, avg_data =0,  minexpt = minexpt
        
        hxi_rebindata, data1_in=tot_counts, data2_in=tot_counts_bg,expt_in=expt,data1_out=tot_counts_new, data2_out=tot_counts_bg_new,$
                expt_out=expt_out, avg_data =0,  minexpt = minexpt
        
        hxi_rebindata, data1_in=cent_xy_FOV,  expt_in=expt,data1_out=cent_xy_FOV_new, $
                expt_out=expt_out, avg_data =1,  minexpt = minexpt

        counts=counts_new
        counts_bg = counts_bg_new
        tot_counts  =  tot_counts_new
        tot_counts_bg  = tot_counts_bg_new
        cent_xy_FOV = cent_xy_FOV_new
        
        ;240201,YS
        if total(expt) le 20. then counts_bg = f_div( counts_bg, counts_bg)* rebin( total(counts_bg,2)/total(expt), 91, n_elements(expt_out),/sample) *transpose( rebin(expt_out ,n_elements(expt_out), 91, /sample),[1,0])
        
        totflux =  total(tot_counts)-total(tot_counts_bg)

        counts_img = total(counts[det_id,*])-total(counts_bg[det_id,*]) ;230418, only consider the selected detectors
        if counts_img lt 5000. then message, 'Warning! The total counts may not be enough for imaging (<5000)',/con
        
        ds=hxi_get_config_data()
        pitches = ds.grid_p *0.0001
        sr=   atan(pitches[0:90] *1.d-2,1.19-0.00 )*180./!pi*3600./2.
        
        ;240310, YS
        maxshiftx = abs( max(cent_xy_FOV[0,*])-min(cent_xy_FOV[0,*]) )
        maxshifty = abs( max(cent_xy_FOV[1,*])-min(cent_xy_FOV[1,*]) )
        if maxshiftx le maxshift and maxshifty le maxshift then no_pot_corr=1
        
        if no_pot_corr eq 1 then begin
          counts = total(counts,2)
          counts_bg = total(counts_bg, 2)
         endif 

        this_pointing = cent_xy_FOV[*,0]  ;240108, YS; 240309, YS, remove'if i eq 0 then'

        if hxi_fov then begin
          this_pointing = [xc,yc]
        endif

        hxi_get_srm4user,earray[*,j], drm_dia, /dia,power=-4.
        p2c = mean( drm_dia ) ;231223,YS
 ;------------------------------------------------------------------------------
    if  STRUPCASE(method) eq 'VIS_BP' or STRUPCASE(method) eq  'VIS_CLEAN' or STRUPCASE(method) eq 'MEM_GE' then begin
     
        vis_hxi=hxi_get_vis( tarray_obs[*,i], earray[*,j],counts,counts_bg,tot_counts,tot_counts_bg,xc=reform(this_pointing[0]),yc=reform(this_pointing[1]),cent_xy=cent_xy_FOV, $  ;yang, 0925, 1017
          det_id=det_id, no_pot_corr=no_pot_corr) 

        hxi_maps[j,i].Time = tarray[0,i]
        hxi_maps[j,i].dur = this_exptime
        hxi_maps[j,i].ID = 'ASO-S/HXI ' +STRUPCASE(method)+ blank(2)+ string(earray[0,j],format='(F5.1)')+' -'+string(earray[1,j],format='(F5.1)')+' keV'
        hxi_maps[j,i].ALGORITH = STRUPCASE(method)
        hxi_maps[j,i].energy_l = earray[0,j]
        hxi_maps[j,i].energy_h = earray[1,j]
        hxi_maps[j,i].p2c = p2c
        hxi_maps[j,i].detid_mm =  detid_mm
        
        ;221016,yang
        hxi_maps[j,i].xc = reform(this_pointing[0])
        hxi_maps[j,i].yc = reform(this_pointing[1])
  
        beam_w= reform(sr[det_id[0]] )*2./2.355/bwf  ;240309,YS
 
        case STRUPCASE(method) of
          'VIS_BP':  begin
            hxi_vis_bpmap, vis_hxi,  BP_FOV=nxy*psize, MAP=rimage,PIXEL=psize,xc=reform(this_pointing[0]),yc=reform(this_pointing[1]),$
              loopstyle=1 ;,data_only=0;,time=systim();,uniform_weighting=1;, spatial_freqency_weight =spatial_frequency_weight
          end
          'VIS_CLEAN':    begin
            hxi_vis_clean, vis_hxi, rimage,xc=reform(this_pointing[0]),yc=reform(this_pointing[1]), $
               image_dim=nxy,pixel=psize, beam_width =beam_w,uniform_weighting=uniform_weighting
          end
          'MEM_GE':  begin
            ;rimage = mem_ge(vis_hxi, totflux, imsize=[nxy,nxy], pixel = [psize,psize])
;            ;240309, YS, under testing: use hxi_mem_ge developed by Paolo Massa to replace mem_ge, 
            hxi_mem_ge, vis_hxi, imsize=[nxy,nxy], pixel=[psize,psize], total_flux=totflux, percent_lambda=percent_lambda,$
              makemap=0, map=rimage, silent=silent, _extra=extra

          end
          Else: message, 'no such imaging algorithm'
        endcase

        ;        if typename(rimage) eq 'INT' then begin
        ;          hxi_maps = 0
        ;          return
        ;        endif
        
        ;2023,April 09, Yang Su
        if STRUPCASE(method) eq 'VIS_CLEAN' or STRUPCASE(method) eq 'MEM_GE' then begin
          rec_image[*,*,j,i]= rimage* f_div( totflux, total(rimage))  /(float(psize)^2.)/p2c
        endif

        if STRUPCASE(method) eq 'VIS_BP' then begin
          ;totflux =  total(counts[-3:-1,*]-counts_bg[-3:-1,*])/3.*4.  
          rec_image[*,*,j,i]= rimage /(float(psize)^2.)/p2c
        endif

        time_title[j,i]=  tarray_obs[0,i]
        hxi_maps[j,i].data= reform(rec_image[*,*,j,i])
        hxi_maps[j,i].time= time_title[j,i]
        hxi_maps[j,i].counts = double( counts_img )
        hxi_maps[j,i].bkg = double(total(counts_bg[det_id, *])) ;230418, yang su

        if hxi_fov eq 1  then begin
          hxi_maps[j,i].xc = xc
          hxi_maps[j,i].yc = yc
        endif else begin
   
            cen_theta =  atan(yc,xc) -pa/180.*!pi
            xc_new = sqrt(xc^2.+yc^2.)*cos(cen_theta)
            yc_new = sqrt(xc^2.+yc^2.)*sin(cen_theta)
 
            hxi_maps[j,i].xc = xc_new
            hxi_maps[j,i].yc = yc_new
            ;temp_map= rot_map(hxi_maps[j,i], -pa,RCENTER=[sdc_in_fov_x ,sdc_in_fov_y ], /ALIGN_ROLL)
            temp_map= rot_map(hxi_maps[j,i], -pa,RCENTER=[0 ,0. ], /ALIGN_ROLL)
            temp_map.roll_center =[0.,0.]
            temp_map.roll_angle = 0.
            smap=extract_map(temp_map,xrange=xc+[-nxy/2*psize,nxy/2*psize],yrange=yc+[-nxy/2*psize,nxy/2*psize],dimensions=dimensions)
            hxi_maps[j,i]= smap

        endelse

        ;hxi_maps[j,i].id = 'ASO-S/HXI | '+ STRUPCASE(METHOD)+' | '+ blank(2) + trim(earray[0,j])+'-'+trim(earray[1,j])+' keV'
  
    endif

    if  STRUPCASE(method) eq 'HXI_BACKPROJ' or STRUPCASE(method) eq  'HXI_CLEAN' then begin
 
        ;hxi_get_pointing, tarray, sdc_pos, rolla

        hxi_get_pattern4imaging,  earray[*,j], this_pointing, fov, rolla, patt_psize, $
          nxy,nocalib, patt,fulld=fulld,cent_xy=cent_xy_fov, patt_shift=patt_shift
 
        ;hxi_get_resp4imaging, earray, resp_diag

        hxi_maps[j,i].Time = tarray_obs[0,i]
        hxi_maps[j,i].dur = this_exptime
        hxi_maps[j,i].ID = 'ASO-S/HXI ' + STRUPCASE(method)+' | '+ string(earray[0,j],format='(F6.1)')+' -'+string(earray[1,j],format='(F6.1)')+' keV'
        hxi_maps[j,i].ALGORITH = STRUPCASE(method)
        hxi_maps[j,i].energy_l = reform(earray[0,j])
        hxi_maps[j,i].energy_h = reform(earray[1,j])
        hxi_maps[j,i].p2c = p2c
        hxi_maps[j,i].detid_mm =  detid_mm   ;240204, YS
        
        ;221016,yang
        hxi_maps[j,i].xc = reform(this_pointing[0])
        hxi_maps[j,i].yc =  reform(this_pointing[1])
 
         if keyword_set(det_bg) then counts_bg = rebin( average( det_bg,2), (size(counts))[1], (size(counts))[2],/sample)
 
      if no_pot_corr eq 1 then begin
        patt_shift =  reform(patt_shift[*,*,*, 0 ])
      endif else begin
         
      endelse
 
    ;counts_bg = counts_bg*0.  ;temp for 0106
    ; temp for 20230112+
;    findind = where(det_id ne 81)
;    if findind[0] ne -1 then det_id = det_id[findind]

        case STRUPCASE(method) of
          'HXI_BACKPROJ':  begin
            HXI_Backproj, (reform(counts)-counts_bg) >0.,this_exptime, patt_shift, totf=counts_img, $
              det_id=det_id,psize=psize,xc=xc, yc=yc, nxy=nxy,no_pot_corr=no_pot_corr, bp_image= rimage
          end
          'HXI_CLEAN':    begin
            HXI_Clean, (reform(counts)-counts_bg) >0.,this_exptime,  patt_shift, totf=counts_img,  $
              psize=psize,xc=xc, yc=yc, nxy=nxy,bwf= bwf, $
              det_id=det_id,niter=niter,gain_fac=gain_fac,no_pot_corr=no_pot_corr, Clean_image=rimage
          end
          'HXI_FFITTING':  begin

          end
          Else: message, 'no such imaging algorithm',/con
        endcase
        
        ;2023,April 09, Yang Su
        if STRUPCASE(method) eq 'HXI_CLEAN' then begin
          rec_image[*,*,j,i]= rimage* f_div( totflux, total(rimage))  /(float(psize)^2.)/p2c
        endif
        
        if STRUPCASE(method) eq 'HXI_BACKPROJ' then begin
          rec_image[*,*,j,i]= rimage /(float(psize)^2.)/p2c
        endif
        
        time_title[j,i]=  tarray_obs[0,i]

        hxi_maps[j,i].data= reform(rec_image[*,*,j,i])
        hxi_maps[j,i].time= time_title[j,i]
        hxi_maps[j,i].counts =  double( counts_img ) 
        hxi_maps[j,i].bkg = double(total(counts_bg[det_id, *])) ;230418, yang su
 
        if hxi_fov eq 1  then begin
          hxi_maps[j,i].xc = xc
          hxi_maps[j,i].yc = yc
        endif else begin

            cen_theta =  atan(yc,xc) -pa/180.*!pi
            xc_new = sqrt(xc^2.+yc^2.)*cos(cen_theta)
            yc_new = sqrt(xc^2.+yc^2.)*sin(cen_theta)
 
            hxi_maps[j,i].xc = xc_new
            hxi_maps[j,i].yc = yc_new
            ;temp_map= rot_map(hxi_maps[j,i], -pa,RCENTER=[sdc_in_fov_x ,sdc_in_fov_y ], /ALIGN_ROLL)
            temp_map= rot_map(hxi_maps[j,i], -pa,RCENTER=[0 ,0. ], /ALIGN_ROLL)
            temp_map.roll_center =[0.,0.]
            temp_map.roll_angle = 0.
            smap=extract_map(temp_map,xrange=xc+[-nxy/2*psize,nxy/2*psize],yrange=yc+[-nxy/2*psize,nxy/2*psize],dimensions=dimensions)
            hxi_maps[j,i]= smap
; 
        endelse
 
;         if keyword_set(fulld) then begin
;           finalimage = hxi_maps[j,i].data
;           findpeak=where(rimage eq max(rimage))
;           flareloc_p=ARRAY_INDICES(rimage, findpeak[0])
;
;           flareloc= flareloc_p*psize - ([xc, yc]+nxy/2*psize)
 
;         endif
     
    endif
 endfor
 
    if STRUPCASE(method) eq 'HXI_DLA' then begin
      
    endif
 
  endfor
  
end