;+
; Mission     : ASO-S/HXI
; Name        : hxi_get_patt
; Purpose     : get HXI modulation pattern for given parameters
; Category    : imaging
; Syntax      :
; Inputs      :
;       loc      - location of the center of FOV, [xc, yc] in arcsec;
;       fov      - FOV along Solar x or y, a single value in arcsec;
;       psize    - pattern pixel size in arcsec, a single value in arcsec;
;       en_ph    - energy used for calculating pattern
;       [savefile]  - save pattern in SAV file;
;       [filename]  - filename for saved file;
;       [ideal]     - ideal case;
;       [trian]     - 1d response, trian. functions;
;       [stype]     - type of (sin) subcollimators
; Outputs     :
;       patterns  - pattern, in [nxy, nxy, 91]
; Outputs (OPTIONAL):
; History     :
;       2018/03/27, Yang Su, a series of test versions.
;       Aug. 2019, Yang Su, first beta version, developed for imaging simultion;
;       Jul. 2020, Yang Su, beta version v0.8a. modified to call hxi_get_config_data and use true HXI grid parameters.  
;       2020/10/06, Yang Su, beta version v0.8a. removed 'step' (no longer needed).
;       Aug. 2021, v0.95 beta, Yang Su, developed for production of levQ2 FITS;
;       Aug. 2022, Yang Su, add stype for subcollimator (sin) type (TBA);
;       Mar. 18 2023, Yang Su, Wei Chen, add measured parameters
;       Jun.-Dec. 2023, Yang Su, a number of improvements to include calibration parameters.
; Contact     : yang.su@pmo.ac.cn
;-

pro hxi_get_patt, loc, fov,psize, en_ph,patterns, det_index=det_index, filename=filename,$
              savefile=savefile, ideal=ideal,nocalib=nocalib, stype=stype,trian=trian
  delim = path_sep() 
  
  default, loc,[0.,0.]   ;arcsec
  default, psize,1.      ;arcsec
  default, fov,200.      ;arcsec
  default, ideal, 0
  default, en_ph, 30.    ; keV
  default, savefile, 0
  default, stype, 1   ; 20220824, 1 for moving rear grids to -y (positioning angle 0 degree), 2 for moving front grids
  default, nocalib, 0  ; 20230602, if set, calibration para. are not used.

  default, filename,curdir()+delim+'HXI_patterns_v098_'+string(en_ph,format='(F07.2)')+'_keV'+'.sav' ;240112,YS

  patt_nx= fix(fov/psize )
  if patt_nx mod 2 ne 1 then patt_nx= patt_nx+1
  patt_ny=patt_nx
  ds=hxi_get_config_data()
  
      gridd=HXI_GRID_INFO(detail=1)  ;230318, wei chen, yang add 

  indc = ds.grid_c_index;con index
  inds = ds.grid_s_index;sin index
  pitches = ds.grid_p 
  grid_thick =  ds.grid_t ;mm
  gphases= ds.grid_phases 
  sc_len = ds.subcoll_L
  slitw = ds.grid_slitw
  
  ;230907, YS, now add beam test result of slitwidth
  slit_f = gridd.slit_f
  slit_r = gridd.slit_r
  dc_ratio_30 = gridd.ratio_30
  mask30 = gridd.mask_30
  tilt_angle_f = gridd.tilt_angle_f
  tilt_angle_r = gridd.tilt_angle_r
  find2 = where(mask30 eq 2)
  dc_beam_d1d18 = gridd.dc_beam_d1d18
  shift_d1d18 = gridd.shift_d1d18
  needf2 = gridd.needf2
  
  dc_ideal = ( slitw/pitches )^2.
  dc_opt  = (slit_f*1000./pitches )*(slit_r*1000./pitches ) 
  grid_shift = fltarr(91)
  
; 230602,230814,231107,240205,YS
    ratio_diffindc = ( dc_ideal-(dc_opt<dc_ideal))/dc_ideal

   if nocalib eq 1 then begin
    slitw = ds.grid_slitw
    ;slitw =  sqrt(dc_opt )*pitches
    
    tilt_angle_f = tilt_angle_f*0.
    tilt_angle_r = tilt_angle_r*0.

   endif else begin
     
     grid_shift[0:17] = shift_d1d18 
     
     case 1 of
       en_ph le 30. : begin
         ;slitw =  sqrt( (dc_ratio_30 > dc_opt) < dc_ideal ) *pitches  ;;230907,YS
         slitw =  sqrt(dc_ratio_30 < dc_ideal)*pitches
         slitw[0:17] = sqrt( dc_beam_d1d18 ) *pitches[0:17]
         ;somedet =  ( dc_opt[18:*] > dc_ratio_30[18:*]) < dc_ideal[18:*]
         somedet =  ( dc_ratio_30[18:*]) 
         slitw[18:*] = sqrt(somedet)*pitches[18:*]
         
         somedet =  ( dc_ratio_30[58:*] > 0.24 ) < dc_ideal[58:*]
         slitw[58:*] = sqrt(somedet)*pitches[58:*]
       end
       (en_ph le 70.) and (en_ph gt 30.) : begin
         tempdet =  sqrt(dc_ratio_30 < dc_ideal)
         tempdet[0:17] = sqrt( dc_beam_d1d18 )
         ;tempdet[18:37] =  sqrt( (dc_ratio_30[18:37]+dc_opt[18:37])/2. < dc_ideal[18:37] )
         slitw =  sqrt(( tempdet * (1.+(en_ph-30.)/200.*ratio_diffindc)) < dc_ideal ) *pitches  ;;230907,YS
 
         somedet =  ( dc_ratio_30[18:*])
         slitw[18:*] = sqrt( ( ( somedet*(1.+(en_ph-30.)/200.*ratio_diffindc[18:*]) ) < dc_ideal[18:*] ) ) *pitches[18:*]

         somedet =  ( dc_ratio_30[58:*] > 0.24 ) < dc_ideal[58:*]
         slitw[58:*] = sqrt( ( ( somedet*(1.+(en_ph-30.)/200.*ratio_diffindc[58:*]) ) < dc_ideal[58:*] ) )*pitches[58:*]
       end
       en_ph gt 70. : begin
         tempdet =  sqrt(dc_ratio_30 < dc_ideal)
         tempdet[0:17] = sqrt( dc_beam_d1d18 )
         ;tempdet[18:37] =  sqrt( (dc_ratio_30[18:37]+dc_opt[18:37])/2. < dc_ideal[18:37] )
         slitw = sqrt( (tempdet* (1.+(en_ph-70.)/200.)) < dc_ideal  ) *pitches

         somedet =  ( dc_ratio_30[18:*])
         slitw[18:*] = sqrt( ( ( somedet*(1.+(en_ph-70.)/200.*ratio_diffindc[18:*]) ) < dc_ideal[18:*] ) ) *pitches[18:*]

         somedet =  ( dc_ratio_30[58:*] > 0.24 ) < dc_ideal[58:*]
         slitw[58:*] = sqrt( ( ( somedet*(1.+(en_ph-70.)/200.*ratio_diffindc[58:*]) ) < dc_ideal[58:*] ) )*pitches[58:*]
       end
     endcase

   endelse

  Angles = ds.grid_angles

  gtypes = fix(angles*0.)
    gtypes[indc]=0
    gtypes[inds]=1
    gtypes[-3:-1]=-1
  
  ;ideal case
  if ideal eq 1 then begin
    den=19.*1.d8 ;g/cm³
    grid_thick= fltarr(n_elements(grid_thick))*1.d-3 
    slitw = pitches/2.
  endif

  n_grids=n_elements(pitches)
  
  ;ideal case,230814, YS
  if nocalib eq 1 then begin
    delta_p=fltarr(91)
  endif else begin
      
      ;load imaging calibration results; test version;
      restore,getenv('hxi_dbase') +delim +'grid'+delim+'dph_v1largeslit_postilt.sav'; 
 
     delta_p=dph_test

     case 1 of
       en_ph le 30. : begin
         delta_p =  delta_p ;;230918,YS
       end
       (en_ph le 40.) and (en_ph gt 30.) : begin
          delta_p =  delta_p*3./4. ;;230918,YS, test, temp soution
       end
       (en_ph le 70.) and (en_ph gt 40.) : begin
         delta_p =  delta_p/2. ;;230918,YS, test, temp soution
       end
       en_ph gt 70. : begin
         delta_p =  delta_p/2. ;;230918,YS
       end
       en_ph gt 120. : begin
         delta_p =  delta_p/4. ;;230918,YS
       end
     endcase
;     
    ;delta_p=fltarr(91)
    ;delta_p[1:88:2] = 10.chen
  endelse
 
  ;=====================================================
  
  patterns= fltarr(patt_nx,patt_ny,91)
  trian= fltarr( 2400/psize+1, 91)
      
  cent=loc
  
  if nocalib eq 1 then begin ; 231101, YS, deal with calibration para.
    sametype=[4,5,5,5,5, 5,5,5,3,2]
    ntype= n_elements(sametype)
    for nn=0,  ntype-1 do begin
      if nn eq 0 then ind0=0 else ind0=total( sametype[0:(nn-1)>0])
      if nn eq 0 then ind1=0 else ind1=total( sametype[0:(nn-1)>0])*2

      HXI_mk_pattern, pit=pitches[ind1],slitw=slitw[ind1], $
        thick=grid_thick[ind1],grid_shift=grid_shift[ind1],$
        needf2_input = needf2[ind1],delta_p=delta_p[indc[ind0+indgen(sametype[nn])]],$
        pos_angle=angles[indc[ind0+indgen(sametype[nn])]],$
        gtype=gtypes[indc[ind0]],gphases=gphases[indc[ind0]],$
        en_for_up=en_ph,fov=fov, cent=cent,psize=psize, den=den,$
        sc_len=sc_len,stype=stype,patt,trian_center

      patterns[*,*,indc[ind0+indgen(sametype[nn])]]=  patt
      trian[*, indc[ind0+indgen(sametype[nn])]] = trian_center

      HXI_mk_pattern, pit=pitches[ind1],slitw=slitw[ind1], $
        thick=grid_thick[ind1],grid_shift=grid_shift[ind1],$
        needf2_input = needf2[ind1],delta_p=delta_p[inds[ind0+indgen(sametype[nn])]],$
        pos_angle=angles[inds[ind0+indgen(sametype[nn])]],$
        gtype=gtypes[inds[ind0]],gphases=gphases[indc[ind0]],$
        en_for_up=en_ph,fov=fov, cent=cent,psize=psize, den=den,$
        sc_len=sc_len, stype=stype,patt,trian_center

      patterns[*,*,inds[ind0+indgen(sametype[nn])]]=  patt
      trian[*,inds[ind0+indgen(sametype[nn]) ]] = trian_center

    endfor

    for mm=n_grids-3,  n_grids-1 do begin

      HXI_mk_pattern, pit=pitches[mm],slitw=slitw[mm],$
        thick=grid_thick[mm],grid_shift=grid_shift[mm],needf2_input = needf2[mm],delta_p=delta_p[mm],$
        pos_angle=angles[mm],gtype=gtypes[mm] , gphases=gphases[mm], $
        en_for_up=en_ph,fov=fov, cent=cent,psize=psize, den=den,sc_len=sc_len,$
        stype=stype, patt,trian_center

      patterns[*,*,mm]=  patt
      trian[ *,mm] = trian_center

    endfor
  endif else begin
    if keyword_set(det_index) then begin
      n_det = n_elements(det_index)
      for i_det = 0, n_det-1 do begin
        tilt = [tilt_angle_f[det_index[i_det]], tilt_angle_r[det_index[i_det]]]
        HXI_mk_pattern, pit=pitches[det_index[i_det]],slitw=slitw[det_index[i_det]], $
          thick=grid_thick[det_index[i_det]],grid_shift=grid_shift[det_index[i_det]],$
          needf2_input = needf2[det_index[i_det]],delta_p=delta_p[det_index[i_det]],$
          pos_angle=angles[det_index[i_det]],$
          gtype=gtypes[det_index[i_det]],gphases=gphases[det_index[i_det]], tilt=tilt,$
          en_for_up=en_ph,fov=fov, cent=cent,psize=psize, den=den,$
          sc_len=sc_len,stype=stype,patt,trian_center

        patterns[*,*,det_index[i_det]]=  patt
        trian[ *,det_index[i_det]] = trian_center
 
      endfor
    endif else begin
      for i_det = 0, 90 do begin
        tilt = [tilt_angle_f[i_det], tilt_angle_r[i_det]]
        HXI_mk_pattern, pit=pitches[i_det],slitw=slitw[i_det], $
          thick=grid_thick[i_det],grid_shift=grid_shift[ i_det ],$
          needf2_input = needf2[i_det],delta_p=delta_p[i_det],$
          pos_angle=angles[i_det],$
          gtype=gtypes[i_det],gphases=gphases[i_det], tilt=tilt,$
          en_for_up=en_ph,fov=fov, cent=cent,psize=psize, den=den,$
          sc_len=sc_len,stype=stype,patt,trian_center

        patterns[*,*,i_det]=  patt
        trian[ *,i_det] = trian_center
        ;print, i_det
      endfor
    endelse

  endelse
 
  grid_para=ds

  if savefile eq 1 then begin
    save, grid_para, patterns, trian, psize, fov,  loc, en_ph,delta_p, filename=filename
  endif
  
end