;+
; Mission     : ASO-S/HXI
; Name        : hxi_mk_pattern
; Purpose     : calculate modulation pattern of HXI subcollimator for the given parameters
; Category    : imaging
; Syntax      :
; Inputs      :
;       
; Outputs     :
;        
; Outputs (OPTIONAL):
; History     :
;       2018/03/27, v0.4, Yang Su, a series of test versions.
;       Dec. 2018, v0.5a, Yang Su, circles are obtained from a mask on the grids in square;
;                 Wu trans. rate is obtained from G4 (Wei Liu);
;       Jan. 2019, v0.5b, for sin grids, the front stays the same, the rear girds move to the left        
;       Sep. 2019, v0.6, Yang Su, for the final configration; can get pattern for the grids with same type and pitchs from a single calling
;       Oct. 2020, v0.8a, Yang Su, modified to use true HXI grid parameters. added slitw.
;       Aug. 2021, v0.95 beta, Yang Su, added input en_for_up;
;       Aug. 2022, v1.0 beta, Yang Su, add stype for subcollimator (sin) type (TBA);
;       Sep.-Dec. 2023, Yang Su, added calculation of pattern with parameters 
;                       from preliminary imaging calibration (tilt, delta_p, grid_shift, etc); 
;                       improved the calculation of pattern (nbins_per_pitch is no longer a constant setting for all grids; trian_center is corrected for phase shift first and then converted to a 2D pattern ) 
;       Jan. 2024, YS, patterns were calculated for the exact locations of each pixel, now they are averaged valued based on pixel size;
;                      add keyword 'fast' for fast calculation (still slow, will work on the analytical)
; Contact     : yang.su@pmo.ac.cn 
;-

pro HXI_mk_pattern, pit=pit, slitw=slitw,thick=thick,grid_shift=grid_shift,needf2_input = needf2_input, $
                  delta_p=delta_p,pos_angle=pos_angle,$
                  gtype=gtype,gphases=gphases,tilt = tilt, en_for_up=en_for_up,fov=fov, $
                  cent=cent,psize=psize, den=den,sc_len=sc_len, stype=stype, fast=fast,$ 
                  patt, trian_center

  ;default gtype, 0  ; 0 for sin, 1 for cos, -1 for 120 degree
  if not keyword_set(gtype) then gtype=0
  if not keyword_set(pit)  then pit=120.
  if not keyword_set(slitw)  then slitw= pit/2.
  if not keyword_set(fov) then fov=101
  if not keyword_set(cent) then cent=[0.,0.]
  if not keyword_set(psize) then psize=1.
  if not keyword_set(thick) then thick=2. ;mm
  if not keyword_set(en_for_up) then en_for_up=30. ; keV
  if not keyword_set(stype) then stype=1 ; 20220824
  if not keyword_set(delta_p) then delta_p=0. ; 20220824; for corrections of grid phase
  if not keyword_set(needf2_input) then needf2_input = 0  ;231117, YS
  if not keyword_set(fast) then fast = 1 ;240107, YS
  
  psize_1st = 1. ;always calculate grid repsonse with pixel size of 1.; 20240105, YS
  
  stype=2 
  
  default, pos_angle,  0.
  default, den, 19.35 ;g/cm³
;  default, sc_len_default, (hxi_constant(/COLLIMATOR_LENGTH)+thick)/10. ;20240104,YS
;  if keyword_set(sc_len) then sc_len_in =sc_len+ thick/10.  else sc_len_in = sc_len_default  
 
  default, sc_len_default,(hxi_constant(/COLLIMATOR_LENGTH) )/10. ;20240122,YS      ;hxi_constant(/COLLIMATOR_LENGTH)/10.
  if keyword_set(sc_len) then sc_len_in =sc_len else sc_len_in = sc_len_default
   
  delim   = path_sep()
  
    ;yang , 221002 , 240104
  maxdis = sqrt(cent[0]^2.+(cent[1]^2.))+fov/2
  if maxdis ge 800. then begin
    np = ( fix( (((fix(maxdis*2.*sqrt(2.)/100.)+1)*100)>2400) /psize)+1 )  ;240309, YS, fixed a bug  
    np_1st = long( np*psize/psize_1st )  ;240309,YS, fix-> long
  endif else begin
    np = fix(2400./psize)+1
    np_1st = long( np*psize/psize_1st ) ;240309,YS, fix-> long
  endelse
  nxy=long(fov/psize)  ;240309,YS, fix-> long
  if nxy mod 2 ne 1 then nxy= nxy+1

  gthick=thick*0.1    ; mm -> cm
 
  needf2 = needf2_input
  gthick_thelayer = 0.01 
  gthick_therest = gthick - gthick_thelayer -  gthick_thelayer *needf2 
 
  nx=long(np)
  ny=long(np)
  nx_1st=long(np_1st)
  ny_1st=long(np_1st)
  ;tri_fun=fltarr(np)
 
  p=pit*0.0001  ;pitch:  p   RHESSI D4 176.990 um ,  0.0176990 cm
  grid_rad_f= 36.*0.1/2.  ;sqrt(36./!pi)  ;cm
  grid_rad_r= 22.*0.1/2.   ;sqrt(22./!pi)  ;cm
  sr=   atan(p  *1.d-2,sc_len_in/100.-0.00 )*180./!pi*3600./2.
  
  scale=[psize_1st,psize_1st]
  nlayer=100
  nbins_per_pitch=200* round(sqrt( pit[0]/36.))  ;231215, YS

  if fast eq 1 then begin
    nlayer=50
    nbins_per_pitch=100* round(sqrt( pit[0]/36.))  ;231215, YS
  endif
  ;;get en-up curve for W
  restore, getenv('hxi_dbase') +delim +'grid'+delim+ 'up_W.sav'  ;221015; 230519, YS, ->hxi_dbase
  
  up_for_this_en = INTERPOL(up,en_up, en_for_up)
 
  den_W=den
  
  ;den_W=19.*1.d8
 
  n_pitch_f=fix((grid_rad_f-p/4.)/(p)+1)*2  
  n_pitch_r=fix((grid_rad_r-p/4.)/(p)+1)*2   ;(+p/2.)

  L_slit= slitw*0.0001 
  L_slat=p -L_slit
  
  ;test,231105,YS
  if keyword_set(tilt) then begin
    the_angle_f =  tilt[0]
    the_angle_r =  tilt[1]
  endif else begin
    the_angle_f = 0.
    the_angle_r = 0.
  endelse

;  L_slit_f= slitw-2*sin(the_angle/180.*!pi)
;  L_slit_r= slitw-2*sin(the_angle/180.*!pi) 

  y0_r_1p= findgen(nbins_per_pitch)*p/float(nbins_per_pitch)

  y0_r_edges = [ -reverse(findgen(n_pitch_r+1)*p/2. +p/4.),findgen(n_pitch_r+1)*p/2. +p/4.]        ;[findgen( (long(n_pitch_r)*2+2))-(n_pitch_r*2)/2.]*p/2. +p/4.
  y0_r_center = [ -reverse(findgen(n_pitch_r)*p/2. +p/2.),0., findgen(n_pitch_r)*p/2. +p/2.]

  y0_r =   rebin( y0_r_edges, long( n_elements(y0_r_edges) )*nbins_per_pitch/2 )    ;  INTERPOLATE()  congrid
  y0_r=  y0_r[0: ( (long( n_elements(y0_r_edges) )-1)*nbins_per_pitch/2) ]

  y0_f_edges = [ -reverse(findgen(n_pitch_f+1)*p/2. +p/4.),findgen(n_pitch_f+1)*p/2. +p/4.] ; [findgen( (long(n_pitch_f)*2+2))-(n_pitch_f*2)/2.]*p/2. +p/4.
  y0_f_center = [ -reverse(findgen(n_pitch_f)*p/2. +p/2.),0., findgen(n_pitch_f)*p/2. +p/2.]  ; [findgen( (long(n_pitch_f)*2+1))-(n_pitch_f*2)/2.]*p/2. +p/4.

  y0_f =  rebin( y0_f_edges, long( n_elements(y0_f_edges) )*nbins_per_pitch/2 )    ;
  y0_f=  y0_f[0: ( (long( n_elements(y0_f_edges) )-1)*nbins_per_pitch/2)  ]
  
  y0_f = y0_f  
  y0_f_edges = y0_f_edges  
  y0_f_center = y0_f_center 
  
  shiftofthelayer = grid_shift * p
  y0_f1 = y0_f  ;test, YS, 231113 
  y0_f2 = y0_f  ;test, YS, 231113 
  y0_r1 = y0_r + shiftofthelayer    ;test, YS, 231113
  
  case stype of
    1: begin
        case gtype of
        0:
        1:  begin
          y0_r=y0_r - p/4.
          y0_r_edges = y0_r_edges - p/4.
          y0_r_center = y0_r_center -p/4.
        end
        -1:  begin
          y0_r=y0_r - gphases[0]/360.*p
          y0_r_edges = y0_r_edges - gphases[0]/360.*p
          y0_r_center = y0_r_center - gphases[0]/360.*p
        end
      endcase
      end
    2: begin
        case gtype of
        0:  begin
              y0_f1=y0_f1 + shiftofthelayer  ;test, 231113, YS
              y0_f2=y0_f2 - shiftofthelayer  ;test, 231113, YS
            end
        1:  begin
          y0_f=y0_f - p/4.  
          y0_f_edges = y0_f_edges - p/4.  
          y0_f_center = y0_f_center -p/4. 
          
          y0_f1=y0_f1 - p/4. + shiftofthelayer  ;test, 231113, YS
          y0_f2=y0_f2 - p/4. - shiftofthelayer  ;test, 231113, YS
        end
        -1:  begin
          y0_f=y0_f - gphases[0]/360.*p 
          y0_f_edges = y0_f_edges - gphases[0]/360.*p   
          y0_f_center = y0_f_center - gphases[0]/360.*p 
          
          y0_f1=y0_f1 - gphases[0]/360.*p  +shiftofthelayer  ;test, 231113, YS
          y0_f2=y0_f2 - gphases[0]/360.*p  -shiftofthelayer  ;test, 231113, YS  
        end
      endcase
      end    
  endcase
 
  ;start point of the prjected rear grid in the front plane
  ind_RF_st= where(y0_f ge y0_r[0])
  ind_RF_st=ind_RF_st[0]
  
  ;;test, 231113, YS 
  ind_RF_st1= where(y0_f1 ge y0_r[0])
  ind_RF_st1=ind_RF_st1[0]
  
  ind_RF_st2= where(y0_f2 ge y0_r[0])
  ind_RF_st2=ind_RF_st2[0]
 
  x_au= 0.
  y_au=(DINDGEN(ny_1st)- ny_1st/2)  * scale[1]
;  y_au=(DINDGEN(ny)-(ny-1)/2)  * scale[1] - fix(the_angle_f*3600./(sr*2.))*(sr*2.) ; 231104, YS, test
  y_au_f=(DINDGEN(ny_1st)-ny_1st/2)  * scale[1] - fix(the_angle_f*3600./(sr*2.))*(sr*2.) ; 231104, YS, test
  y_au_r=(DINDGEN(ny_1st)-ny_1st/2)  * scale[1] - fix(the_angle_r*3600./(sr*2.))*(sr*2.) ; 231104, YS, test
 

  z0_r= sc_len_in
  z0_f= 0.

  z_r= z0_r   ;y of   the layer
  z_f= z0_f   ;y of   the layer
 
  patt_xcenter=fltarr(np_1st)
  I_total=fltarr(np_1st)

  ;length of the slits and slats in the rear grids
  len_slits_r= 2.* sqrt( ( grid_rad_r^2. - (findgen( (n_pitch_r+1)*2+1)*p/2.+p/4.+y0_r[0])^2. ) > 0. )
  len_slits_r= 2.* sqrt( ( grid_rad_r^2. - (y0_r_center)^2. ) > 0. )
  len_slits_r_all =  rebin(len_slits_r ,  n_elements(y0_r)-1 ,/sample )
 
  ;20201006
  ;len_slits_r_all= 2.* sqrt( ( grid_rad_r^2. - (y0_r)^2. ) > 0. )

  for nny=0, ny_1st-1L do begin
    ang_a= abs( y_au_f[nny]/3600./180.*!pi )
    
    ;test
    L_slit= slitw*0.0001 
    L_slat=p -L_slit
    
    dis_1p= fltarr(nbins_per_pitch)

    if y_au_f[nny] le 0 then begin
      ind1= where( y0_r_1p le gthick_therest*tan(ang_a) )
      if ind1[0] ne -1 then dis_1p[ind1]=  f_div( y0_r_1p[ind1], sin(ang_a) )

      ind2= where( (y0_r_1p ge gthick_therest*tan(ang_a)) and  (y0_r_1p lt L_slat) )
      if ind2[0] ne -1 then dis_1p[ind2]=  f_div( gthick_therest, cos(ang_a) )

      ind3= where( (y0_r_1p ge L_slat) and  (y0_r_1p lt (L_slat+gthick_therest*tan(ang_a) )) )
      if ind3[0] ne -1 then dis_1p[ind3]=  ( f_div( gthick_therest, cos(ang_a) ) - f_div(  y0_r_1p[ind3]-L_slat, sin(ang_a) )  )

      ind4= where( (y0_r_1p ge (L_slat+gthick_therest*tan(ang_a) )) and  (y0_r_1p lt (L_slat+L_slit)) )
      if ind4[0] ne -1 then  dis_1p[ind4]= 0.

    endif else begin
      ind1= where( (y0_r_1p le L_slat-gthick_therest*tan(ang_a)) )
      if ind1[0] ne -1 then dis_1p[ind1]=  f_div( gthick_therest, cos(ang_a) )

      ind2= where( (y0_r_1p ge L_slat-gthick_therest*tan(ang_a)) and (y0_r_1p lt L_slat) )
      if ind2[0] ne -1 then dis_1p[ind2]=   f_div(  L_slat-y0_r_1p[ind2], sin(ang_a) )

      ind3= where( (y0_r_1p ge L_slat) and  (y0_r_1p lt (L_slat+L_slit-gthick_therest*tan(ang_a) )) )
      if ind3[0] ne -1 then dis_1p[ind3]= 0.

      ind4= where( (y0_r_1p ge (L_slat+L_slit-gthick_therest*tan(ang_a) ) ) and  (y0_r_1p lt (L_slat+L_slit)) )
      if ind4[0] ne -1 then  dis_1p[ind4]= ( gthick_therest/cos(ang_a)- (L_slat+L_slit-y0_r_1p[ind4])/sin(ang_a) )     ;0.

    endelse
 
    dis_f= reform(  rebin( dis_1p , nbins_per_pitch,long(n_pitch_f+1),/sample ), nbins_per_pitch*long(n_pitch_f+1))
    dis_f=dis_f[nbins_per_pitch/2:*]
    
    ;;;-----------------------------------------------------
          dis_1p= fltarr(nbins_per_pitch)
 
          if y_au_f[nny] le 0 then begin
            ind1= where( y0_r_1p le gthick_thelayer*tan(ang_a) )
            if ind1[0] ne -1 then dis_1p[ind1]=  f_div( y0_r_1p[ind1], sin(ang_a) )
      
            ind2= where( (y0_r_1p ge gthick_thelayer*tan(ang_a)) and  (y0_r_1p lt L_slat) )
            if ind2[0] ne -1 then dis_1p[ind2]=  f_div( gthick_thelayer, cos(ang_a) )
      
            ind3= where( (y0_r_1p ge L_slat) and  (y0_r_1p lt (L_slat+gthick_thelayer*tan(ang_a) )) )
            if ind3[0] ne -1 then dis_1p[ind3]=  ( f_div( gthick_thelayer, cos(ang_a) ) - f_div(  y0_r_1p[ind3]-L_slat, sin(ang_a) )  )
      
            ind4= where( (y0_r_1p ge (L_slat+gthick_thelayer*tan(ang_a) )) and  (y0_r_1p lt (L_slat+L_slit)) )
            if ind4[0] ne -1 then  dis_1p[ind4]= 0.
      
          endif else begin
            ind1= where( (y0_r_1p le L_slat-gthick_thelayer*tan(ang_a)) )
            if ind1[0] ne -1 then dis_1p[ind1]=  f_div( gthick_thelayer, cos(ang_a) )
      
            ind2= where( (y0_r_1p ge L_slat-gthick_thelayer*tan(ang_a)) and (y0_r_1p lt L_slat) )
            if ind2[0] ne -1 then dis_1p[ind2]=   f_div(  L_slat-y0_r_1p[ind2], sin(ang_a) )
      
            ind3= where( (y0_r_1p ge L_slat) and  (y0_r_1p lt (L_slat+L_slit-gthick_thelayer*tan(ang_a) )) )
            if ind3[0] ne -1 then dis_1p[ind3]= 0.
      
            ind4= where( (y0_r_1p ge (L_slat+L_slit-gthick_thelayer*tan(ang_a) ) ) and  (y0_r_1p lt (L_slat+L_slit)) )
            if ind4[0] ne -1 then  dis_1p[ind4]= ( gthick_thelayer/cos(ang_a)- (L_slat+L_slit-y0_r_1p[ind4])/sin(ang_a) )     ;0.
      
          endelse
      
          dis_f1= reform(  rebin( dis_1p , nbins_per_pitch,long(n_pitch_f+1),/sample ), nbins_per_pitch*long(n_pitch_f+1))
          dis_f1=dis_f1[nbins_per_pitch/2:*]
    
    ;;;--------------------------------------------------------
    
    ;;;-----------------------------------------------------

          if needf2 eq 1 then begin
            dis_1p= fltarr(nbins_per_pitch)
            if y_au_f[nny] le 0 then begin
              ind1= where( y0_r_1p le gthick_thelayer*tan(ang_a) )
              if ind1[0] ne -1 then dis_1p[ind1]=  f_div( y0_r_1p[ind1], sin(ang_a) )
              ind2= where( (y0_r_1p ge gthick_thelayer*tan(ang_a)) and  (y0_r_1p lt L_slat) )
              if ind2[0] ne -1 then dis_1p[ind2]=  f_div( gthick_thelayer, cos(ang_a) )
              ind3= where( (y0_r_1p ge L_slat) and  (y0_r_1p lt (L_slat+gthick_thelayer*tan(ang_a) )) )
              if ind3[0] ne -1 then dis_1p[ind3]=  ( f_div( gthick_thelayer, cos(ang_a) ) - f_div(  y0_r_1p[ind3]-L_slat, sin(ang_a) )  )
              ind4= where( (y0_r_1p ge (L_slat+gthick_thelayer*tan(ang_a) )) and  (y0_r_1p lt (L_slat+L_slit)) )
              if ind4[0] ne -1 then  dis_1p[ind4]= 0.
            endif else begin
              ind1= where( (y0_r_1p le L_slat-gthick_thelayer*tan(ang_a)) )
              if ind1[0] ne -1 then dis_1p[ind1]=  f_div( gthick_thelayer, cos(ang_a) )
              ind2= where( (y0_r_1p ge L_slat-gthick_thelayer*tan(ang_a)) and (y0_r_1p lt L_slat) )
              if ind2[0] ne -1 then dis_1p[ind2]=   f_div(  L_slat-y0_r_1p[ind2], sin(ang_a) )
              ind3= where( (y0_r_1p ge L_slat) and  (y0_r_1p lt (L_slat+L_slit-gthick_thelayer*tan(ang_a) )) )
              if ind3[0] ne -1 then dis_1p[ind3]= 0.
              ind4= where( (y0_r_1p ge (L_slat+L_slit-gthick_thelayer*tan(ang_a) ) ) and  (y0_r_1p lt (L_slat+L_slit)) )
              if ind4[0] ne -1 then  dis_1p[ind4]= ( gthick_thelayer/cos(ang_a)- (L_slat+L_slit-y0_r_1p[ind4])/sin(ang_a) )     ;0.
            endelse
            dis_f2= reform(  rebin( dis_1p , nbins_per_pitch,long(n_pitch_f+1),/sample ), nbins_per_pitch*long(n_pitch_f+1))
            dis_f2=dis_f2[nbins_per_pitch/2:*]
          endif
           
    ;;;--------------------------------------------------------
    
    ang_b= abs( y_au_r[nny]/3600./180.*!pi )
    dis_1p= fltarr(nbins_per_pitch)
    
    ;test
    L_slit= slitw*0.0001
    L_slat=p -L_slit

    if y_au_r[nny] le 0 then begin
      ind1= where( y0_r_1p le gthick*tan(ang_b) )
      if ind1[0] ne -1 then dis_1p[ind1]=  f_div( y0_r_1p[ind1], sin(ang_b) )

      ind2= where( (y0_r_1p ge gthick*tan(ang_b)) and  (y0_r_1p lt L_slat) )
      if ind2[0] ne -1 then dis_1p[ind2]=  f_div( gthick, cos(ang_b) )

      ind3= where( (y0_r_1p ge L_slat) and  (y0_r_1p lt (L_slat+gthick*tan(ang_b) )) )
      if ind3[0] ne -1 then dis_1p[ind3]=  ( f_div( gthick, cos(ang_b) ) - f_div(  y0_r_1p[ind3]-L_slat, sin(ang_b) )  )

      ind4= where( (y0_r_1p ge (L_slat+gthick*tan(ang_b) )) and  (y0_r_1p lt (L_slat+L_slit)) )
      if ind4[0] ne -1 then  dis_1p[ind4]= 0.

    endif else begin
      ind1= where( (y0_r_1p le L_slat-gthick*tan(ang_b)) )
      if ind1[0] ne -1 then dis_1p[ind1]=  f_div( gthick, cos(ang_b) )

      ind2= where( (y0_r_1p ge L_slat-gthick*tan(ang_b)) and (y0_r_1p lt L_slat) )
      if ind2[0] ne -1 then dis_1p[ind2]=   f_div(  L_slat-y0_r_1p[ind2], sin(ang_b) )

      ind3= where( (y0_r_1p ge L_slat) and  (y0_r_1p lt (L_slat+L_slit-gthick*tan(ang_b) )) )
      if ind3[0] ne -1 then dis_1p[ind3]= 0.

      ind4= where( (y0_r_1p ge (L_slat+L_slit-gthick*tan(ang_b) ) ) and  (y0_r_1p lt (L_slat+L_slit)) )
      if ind4[0] ne -1 then  dis_1p[ind4]= ( gthick/cos(ang_b)- (L_slat+L_slit-y0_r_1p[ind4])/sin(ang_b) )     ;0.

    endelse
    
    dis_r= reform(  rebin( dis_1p , nbins_per_pitch,long(n_pitch_r+1),/sample ), nbins_per_pitch*long(n_pitch_r+1))
    dis_r=dis_r[nbins_per_pitch/2:*]
 
;    grid_f2sc_y=  round( ( -(z0_r-z_f)* (tan(y_au[nny]/3600./180.*!pi))  ) / (p/nbins_per_pitch) )
;    grid_r2sc_y=  round( ( -(z0_r-z_r)* (tan(y_au[nny]/3600./180.*!pi)) ) / (p/nbins_per_pitch) )

    grid_f2sc_y=  round( ( (z0_r-z_f)* (tan(y_au_f[nny]/3600./180.*!pi))  ) / (p/nbins_per_pitch) )
    grid_r2sc_y=  round( (  (z0_r-z_r)* (tan(y_au_r[nny]/3600./180.*!pi)) ) / (p/nbins_per_pitch) )
    
    ;test, 231113, YS
    grid_f12sc_y=  round( ( (z0_r-z_f+ gthick/2. )* (tan(y_au_f[nny]/3600./180.*!pi))  ) / (p/nbins_per_pitch) )
    if needf2 eq 1 then grid_f22sc_y=  round( ( (z0_r-z_f- gthick/2. )* (tan(y_au_f[nny]/3600./180.*!pi))  ) / (p/nbins_per_pitch) ) 
    
    dis_rr= dis_f > 0.2*2.
    ;rr_slat[ind_RF_st: ind_RF_st+n_elements(r_slat)-1]=r_slat
    dis_rr[ind_RF_st]=dis_r  ; faster
    
    ;test, 231114, YS
;    dis_rr1= dis_f > 0.2*2.
;    dis_rr1[ind_RF_st2]=dis_r1  ; faster

    trans_layers = fltarr( n_elements(y0_r)-1) +0.4

    temp_f= grid_f2sc_y
    temp_r=grid_r2sc_y
    
    temp_f1 = grid_f12sc_y ;231113, YS
 
    ;? trans_layers= dis_f[ ((ind_RF_st-temp_f)) : (ind_RF_st-temp_f+n_elements(dis_r)-1 )  ]  + dis_rr[ (ind_RF_st-temp_r): ( ind_RF_st-temp_r+n_elements(dis_r)-1)]
    ;trans_layers= dis_f[ ((ind_RF_st-temp_f)>0) : (ind_RF_st-temp_f+n_elements(dis_r)-1 )  ]  + dis_rr[ (ind_RF_st-temp_r): ( ind_RF_st-temp_r+n_elements(dis_r)-1)]
    
    ;20201006, bug found during 513test?
    ind0=    (ind_RF_st+temp_f )      
    ind0_pos= abs(ind0<0)
    
    ;test, YS, 231113
    ind1=    (ind_RF_st1+temp_f1 )
    ind1_pos= abs(ind1<0)
    
    if needf2 eq 1 then begin
      temp_f2 = grid_f22sc_y
      ind2=    (ind_RF_st2+temp_f2)
      ind2_pos= abs(ind2<0)
    end  
 
    comp_f = dis_f[ ((ind0)>0) : ( ( ind0 +n_elements(dis_r)-1 ) < (n_elements(dis_f)-1) ) ]
    comp_r = dis_rr[ ((ind0_pos+ind_RF_st)> ind_RF_st) : (  ( ind_RF_st  +n_elements(dis_r)-1)  < (n_elements(dis_f)-1) )]
     comp_f1 = dis_f1[ ((ind1)>0) : ( ( ind1 +n_elements(dis_r)-1 ) < (n_elements(dis_f1)-1) ) ]
 
    temp= comp_f + comp_r 
 
    ;temp[0: ( n_elements(temp)<n_elements(comp_f1) -1 ) ] = temp[0: ( n_elements(temp)<n_elements(comp_f1) -1 ) ] + comp_f1[0: ( n_elements(temp)<n_elements(comp_f1) -1 ) ]
    trans_layers[ind0_pos]=temp[0: ( n_elements(trans_layers)-1-ind0_pos)  < (n_elements(temp)-1)]
    
    trans_layers[ind1_pos]= trans_layers[ind1_pos:*]  + comp_f1 
    if needf2 eq 1 then begin
      comp_f2 = dis_f2[ ((ind2)>0) : ( ( ind2 +n_elements(dis_r)-1 ) < (n_elements(dis_f2)-1) ) ]
      trans_layers[ind2_pos]= trans_layers[ind2_pos:*]  + comp_f2
    endif
    
;    if ind0 lt 0 then begin
;      trans_layers[0:(ind0_pos-1)>0]=0.2*2.
;    endif
;
;    if ind0 gt (n_elements(dis_f)-1)-(n_elements(dis_r)-1) then begin
;      trans_layers[(( n_elements(trans_layers)-1-ind0_pos)  < (n_elements(temp)-1)) +1 : *]=0.2*2.
;    endif
    
    trans= ( exp(-up_for_this_en*double(trans_layers)*den_W) )  ;I/I0= exp(-up*x)
    I_total[nny]= total(  trans *len_slits_r_all) *( p/float( nbins_per_pitch) )
 ;if nny eq 500 or nny ge 1000 then stop
  endfor
 
  patt_xcenter =I_total/!pi/grid_rad_r^2.
  patt_xcenter_rebin = average( reform(patt_xcenter, fix(psize), np ), 1) ;240105,YS

  fullfov=np     ;fix(long(2100)/psize)+1

  na=n_elements(pos_angle)
 
  ;230915,YS
  centp= cent/psize + fullfov/2

  ;240105,YS
  ind_xs= ( ( (centp[0]-fix(nxy/2)))  ) > 0  
  ind_ys= ( ( (centp[1]-fix(nxy/2)))  ) > 0
  
  trian_center=fltarr(fullfov/2+1200./psize - (fullfov/2-1200./psize)+1, na) 
  
  ;patt_temp = fltarr( nx, ny, na )
  patt = fltarr(nxy, nxy, na)
  
  pixels= pixel_coord( [nxy,nxy])
  pixels_x = pixels[0,*]+ind_xs+nxy/2
  pixels_y = pixels[1,*]+ind_ys+nxy/2
 
;  pos_xy= ARRAY_INDICES(reform(patt_temp[*,*,0]), lindgen(long(fullfov)*long(fullfov) ))
;  ind =  where( ( (pos_xy[0,*]-np/2)^2.+ (pos_xy[1,*]-np/2)^2. ) gt (np/2)^2.)
 
  for mm=0, na-1 do begin
 
    ;230915,YS
      this_angle= pos_angle[mm]
      this_shift = delta_p[mm]
 
    ;231215,YS
     this_pattxcenter=  INTERPOL(patt_xcenter_rebin, findgen(fullfov), findgen(fullfov)- this_shift/psize) > 1.e-10
    trian_center[*,mm]= this_pattxcenter[fullfov/2-1200./psize:fullfov/2+1200./psize] 
    
    tempimg= rebin(transpose(this_pattxcenter), nx,ny, /sample)
    tempimg = rot(  (tempimg), -this_angle, 1., /INTERP ) > 1.e-10  ;ROT( A, Angle
    
    ;for nnx=0,nx-1 do tempimg[nnx, *]= tempimg[ nnx,*]/cos( (nnx-nx/2)*psize/3600./180.*!pi)

    patt[*,*,mm] = INTERPOLATE(tempimg, pixels_x, pixels_y)

  endfor
 
end
