;+
; Mission     : ASO-S/HXI
; Name        : HXI_write_imagecube
; Purpose     : write HXI imagecube FITS
; Category    : imaging; data production
; Syntax      : 
; Inputs      :
;       tarray      - input time intervals, [2,ntime ]
;       earray      - input energy ranges [2, nenergy]
;       maps        - HXI image maps
;       info_str    - infor parameters for imaging
;       psize/xc/yc/nxy - map parameters: pixel size/xcenter/ycenter/number of pixels along x(y)
;       [flareid]   - corresponding flareid
;       [levq2]     - if set, produce levQ2 FITS (not released)
;       [filename]  - user defined filename; if not set, use the filename automatically generated;
;       [outdir]    - directory for output file
; Outputs     :
; Outputs (OPTIONAL):
; History     :
;       Jan. 2022, first beta version, Yang Su, developed for production of levQ2 FITS; 
;       Jul. 2022, v1.0 beta, Yang Su, many changes to the code due to the development of simulated data;
;       Aug. 2022, Yang Su, now the maps are made by map2fits; the FITS can be easily converted to maps by fits2map; 
;       2022/09/17, Fanxiaoyu Xia, added windows for file selection, added default filename and error
;                 message for empty filename.
;       2022/09/27, Yang Su, a few minor changes, program header and comments;
; Contact     : yang.su@pmo.ac.cn
;-

pro hxi_write_imagecube, tarray, earray, maps, info_str,  flareid= flareid,$
                  levq2=levq2, filename=filename, outdir=ourdir

  ;( nxy, nxy, n_energy, n_time )
  
  delim = path_sep()
  default, outdir, curdir()
  default, levq2, 0
  
  dir_hxi=getenv('hxi_db')
 
  if levq2 eq 1 then outdir=getenv('hxi_db_levq2')
  
  if (size(tarray))[0] eq 1 then tarray =  reform(tarray, 2,1)
  if (size(earray))[0] eq 1 then earray =  reform(earray, 2,1)
  
  nt= (size(tarray))[2]
  nen= (size(earray))[2]
  
  imdim=  size(maps,/dim)  
  
  nmaps= n_elements(maps)
 
  stdate= UTC2YYMMDD( anytim(tarray[0],/UTC_EXT),/yyyy )
  sttime= UTC2YYMMDD( anytim(tarray[0],/UTC_EXT),/yyyy, /HHMMSS)
  styear= STRMID(stdate,0,4)
  stmon= STRMID(stdate,4,2)
 
  tande= string(nen,format='(I02)')+'e'+string(nt,format='(I02)')+'t'   ;(I-3)
   img_alg= maps.Algorith
  
  ntime = n_elements(tarray)
  nenergy = n_elements(earray)
  
  this_ver =  '01'
  if levq2 eq 1 then begin
    mk_dir, outdir+delim+styear +delim+stmon +delim+'fits'
    filename=outdir+delim+styear +delim+stmon +delim+'fits' +delim+'hxi_evt_levq2_'+flareid+'_'+sttime+'_'+tande+'_'+img_alg[0]+'_v'+this_ver+'.fits'
  endif else begin
    if not keyword_set(filename) then begin
      filename=outdir+delim+'hxi_imgcube_'+tande+'_'+sttime+'_'+img_alg[0]+'.fits'
      filename=dialog_pickfile(FILTER=['*.fits*;*.fts*'],TITLE="Select a fits file",$
        file=filename)
    endif else begin
      filename= filename+'.fits'
    endelse
  endelse
  
  if filename ne '' then begin
 
  ;mk_fid
  
  maps4fits = reform(maps, nen*nt)
     
  map2fits, maps4fits, filename

  header0=headfits(filename)
      
   If levq2 eq 1 then fxaddpar,header0,'DATATYPE','HXI_levq2_imagecube','HXI level Q2 imagecube' $
    else fxaddpar,header0,'DATATYPE','HXI_lev2_imagecube','HXI level 2 imagecube'
   FxAddPar,header0,'ORIGIN','ASO-S'
   fxaddpar,header0,'TELESCOP','ASO-S','Name of spacescraft'
   fxaddpar,header0,'INSTRUME','HXI', 'Name of instrument'
   FxAddPar,header0,'CAMERA','HXI_DET'
 
   FxAddPar,header0,'UNIT','photons cm-2 s-1 asec-2'
   FxAddPar,header0,'EN_UNIT','keV'
   FxAddPar,header0,'TIMEUNIT','s' 
   
   FxAddPar,header0,'OBJECT','Sun', 'Object being observed'
   FxAddPar,header0,'TIME_ST',sttime[0], 'Start nominal U.T. date of the time intervals'
   FxAddPar,header0,'TIME_ED',sttime[-1], 'End nominal U.T. date of the time intervals'
   ;FxAddPar,header,'TIMESYS','1979.00','Reference Time'
 
   FxAddPar,header0,'ERANGE_L',earray[0],'Start energy of the energy bands'
   FxAddPar,header0,'ERANGE_H',earray[-1],'Highest energy of the energy bands'

;   FxAddPar,header,'NX',nen,'Number of energy bands'
;   FxAddPar,header,'NY',nt,'Number of time intervals'
   FxAddPar,header0,'N_IMAGES',nmaps
   FxAddPar,header0,'WAVELNTH','X-ray'
   FxAddPar,header0,'N_ENERGY',nen
   FxAddPar,header0,'N_TIME',nt
   
   GET_DATE,cdate,/timetag
   FxAddPar,header0,'DATE', cdate,'File creation date (YYYY-MM-DDThh:mm:ss UT)'
 
   ;new ext 
   fxhmake,header1,/XTENSION
   If levq2 eq 1 then fxaddpar,header1,'EXTNAME','PARAMETERS','Parameters for HXI level Q2 quciklook images' $
    else fxaddpar,header1,'EXTNAME','PARAMETERS','Parameters for HXI level 2 images'
   mwrfits,info_str,filename,header1, /silent;, no_types=1, no_comment=1

   ;update main header
   MODFITS, filename, 0, header0 , EXTEN_NO =0
   
   PRINT, 'The '+file_basename(filename)+' has been written! '
      ;mwrfits,maps4fits.data, filename ,header,/CREATE, /silent
    ;fxwrite, filename, header,data,append=(i gt 0)
  endif else begin
    a = dialog_message('No filename is given!')
  endelse
 
end