;+
; Mission: 
;    ASO-S/HXI
; Name:
;    HXI_DATA
; Purpose:
;    Read hxi spectrum data for OSPEX
; INPUT:
;    files     -  hxi spectrum files
; OUTPUT:
;    data_str  -  data struct for SPEX
;
; History:
;    2020/07/12, WeiChen, first edition 
;    2022/04/06, WeiChen

pro hxi_data, FILES=files, $
              DATA_STR=data_str, $
              ERR_CODE=err_code, $
              ERR_MSG=err_msg ,$
              _REF_EXTRA=_ref_extra
 
  data_str = -1
  err_code = 0
  err_msg  = ''
  respfile = ' '

  fitsfile=files[0]
  IF ~FILE_TEST (FITSFILE) THEN BEGIN
    err_code=1
    err_msg='Cannot find the input HXI-fits file '+fitsfile
    RETURN
  ENDIF
  spec_dir=file_dirname(fitsfile)  ;;LZT 20231106

  FITS2SPECTRUM,file=fitsfile, $
                PRIMARY_HEADER=p_hdr, $
                EXT_HEADER=sp_hdr, $
                EXT_DATA=sp_data, $
                ENEBAND_DATA=en_data, $
                _EXTRA=_ref_extra, $
                ERR_MSG=err_msg, $
                ERR_CODE=err_code, /silent

 IF ~IS_STRING(P_HDR) THEN BEGIN
   err_code=1
   err_msg='File '+fitsfile +' is not correct HXI_spex file.'
   RETURN
 ENDIF
  ct_edges = TRANSPOSE([[en_data.e_min], [en_data.e_max]])
  ltime    = sp_data.livetime
  rcounts  = sp_data.rate * ltime 
  IF CHKTAG(sp_data, 'stat_err') THEN ercounts = sp_data.stat_err * SQRT(ltime)  ELSE ercounts = SQRT(rcounts)
  
  ut_edge  = [sp_data[0].time-sp_data[0].timedel/2., sp_data.time+sp_data.timedel/2.]
  ut_edges = GET_EDGES(ut_edge,/edges_2)

  start_time= FXPAR(p_hdr,'DATE_OBS')
  end_time  = FXPAR(p_hdr,'DATE_END')
  detused   = FXPAR(p_hdr,'DET_NAM')
  data_name = FXPAR(p_hdr,'INSTRUME')
  area      = FXPAR(p_hdr,'GEOAREA')
  units     = 'Counts'
  title     = 'HXI spectrum'

  drmfile = FITS_KEYWORD_VALUE(p_hdr, 'DRMFILE')
  IF drmfile EQ '' THEN BEGIN
    err_code=1
    err_msg='The input FITS file does not contain the response matrix link.'
    RETURN
  ENDIF
 
 respfile = LOC_FILE( drmfile, COUNT=count, LOC=loc )
 
  
; IF count LE 0 THEN BEGIN 
;   respfile = FILE_SEARCH( FILE_DIRNAME(fitsfile),drmfile, COUNT=dcount )
;   IF dcount GT 0 THEN respfile = respfile[0] ELSE respfile=''
; ENDIF 
 
 IF count GT 0 THEN BEGIN
     respfile = CONCAT_DIR( loc[0], FILE_BASENAME(respfile[0]) )
   ENDIF ELSE BEGIN
     respfile = FILE_SEARCH( FILE_DIRNAME(fitsfile),drmfile, COUNT=dcount )
      IF dcount GT 0 THEN respfile = respfile[0] ELSE BEGIN  
           PRINT,  'RESPFILE: ' + drmfile + ' not found.' 
           respfile = ''
       ENDELSE
 ENDELSE

  data_str={$
          START_TIME: start_time, $      ;start time of file in anytim format *
          END_TIME  : end_time, $        ;end time of file in anytim format *
          RCOUNTS   : rcounts, $         ;[nenergy,ntime] count data *
          ERCOUNTS  : ercounts, $        ;[nenergy,ntime] error in count data *
          UT_EDGES  : ut_edges, $        ;[2,ntime] edges of time bins in anytim format *
          UNITS     : units, $           ;units of rcounts, usually 'counts' *
          AREA      : area, $            ;effective detector area in cm^2 *
          LTIME     : ltime, $           ;[nenergy,ntime] livetimes *
          CT_EDGES  : ct_edges, $        ;[2,nenergy] edges of energy bins in keV *
          DATA_NAME : data_name, $       ;string name of data, e.g. 'HXI' *
          TITLE     : title, $           ;string title of data, e.g. 'HXI SPECTRUM' *
          RESPFILE  : respfile, $        ;string name of corresponding SRM file, or blank string. *
          ;                              OR [nedges_out, nedges_in] numeric array of DRM values
          ;                              OR structure containing fields drm ([nedges_out, nedges_in]), edges_in, and edges_out
          detused   : detused, $         ;string of detector names *
          atten_states: -1, $            ;[ntime] array of filter states, or scalar -1 if doesn't apply *
          deconvolved : 0, $             ;If 1, original data is already photons *
          pseudo_livetime: 0, $          ;if 1, livetime isn't a real livetime *
          xyoffset    : [0, 0]}          ;[2] x,y location of source on Sun *



;   filespec='hxi_20220919_2120_2230_thick_4spex.fits'   ;hxi_20220919_2120_2230_g1_4spex.fits
;   filedrm='hxi_20220919_2120_2230_thick_drm.fits'
;  o=ospex()
;  o->set, spex_file_reader='hxi'
;  o->set, spex_specfile=filespec   ; name of your spectrum file
;  o->set, spex_drmfile=filedrm             ; name of your response file
;  o-> set, spex_fit_time_interval= ['19-Sep-2022 21:40:10.000', '19-Sep-2022 22:30:10.000']

  ;stop
end