;+
; Mission:
;    ASO-S/HXI
; Purpose:
;    Calculate the detector response matrix of 
;     HXI user-defined energy bands
; Inputs:
;    drm  --   hxi drm
;    ct_ener --    array of energy edges of e-bins 
;    ph_ener --    array of energy edges of photons in drm 
;    ct_edges --   user-defined of energy edges of e-bins
;    ph_edges --   user-defined of energy edges of photons in drm 
; Keyword:
;    perbin  --  units of per keV  
;    ph_mean --  input energy is mean energy   
; Output:
;    srm  --     new drm
; History:
;     WeiChen, 2022/01/11
;     WeiChen, 2024/01/26,
;
PRO hxi_extract_srm4user, drm, ct_ener, ph_ener, ct_edges, ph_edges, srm, $
     perbin=perbin,ph_mean=ph_mean, linear=linear, power=power

 power = fcheck(power,-3.5)
 
 if min(ct_edges) lt min(ct_ener) OR max(ct_edges) gt max(ct_ener) OR $
   min(ph_edges) lt min(ph_ener) OR max(ph_edges) gt max(ph_ener) then $
   message, 'The energy bands set is beyond range! ',/info
 
 edge_products,ct_ener,edges_1=ct_ener1, mean=em1
 ;edge_products,ph_ener,edges_2=ph_ener2, mean=em2
 if keyword_set(ph_mean) then em2=ph_ener else em2= get_edges(ph_ener,/mean)

 ndim=size(drm,/dim)
 if n_elements(em1) NE ndim(0) OR n_elements(em2) NE ndim(1) then $
   message, 'The dimension of DRM is wrong! ',/info


edge_products,ct_edges,edges_1=ct_edge1, edges_2=ct_edge2, mean=ectm,width=ectw
edge_products,ph_edges,edges_1=ph_edge1, edges_2=ph_edge2, mean=ephm,width=ephw
  
  nct=n_elements(ectw)
  nph=n_elements(ephw)
  srm =dblarr(nct,nph)
; indy = where_near( em2, ephm)
; temp= drm(*, indy)
  temp= dblarr(ndim(0),nph)
  
 indy= value_locate(ph_ener, ph_edge1)  ;closest_ix(em2, ph_edge1)  ;where_near  ;2024/01/26
   if keyword_set(linear) then begin
     for i=0, nph-1 do begin
      if (indy[i+1]-indy[i]) LT 1 then temp(*,i)=drm(*, indy[i]) else $
           temp(*,i) = average( drm(*, indy[i]:indy[i+1]),2) ;2024/01/26
     endfor
   endif else begin
    weight= transpose(em2) ^ power
    weight= rebin( weight, ndim(0), ndim(1))
     for i=0, nph-1 do begin
      if (indy[i+1]-indy[i]) LE 1 then temp(*,i)=drm(*, indy[i]) else $
      temp(*,i) = f_div( total( drm(*, indy[i]:indy[i+1])* weight(*,indy[i]:indy[i+1]),2), $
            total( weight(*,indy[i]:indy[i+1]),2) )
     endfor
  
   endelse
 
 indx= closest_ix( ct_ener1, ct_edge1)  ; where_near
  for j= 0, nct-1 do  begin
    if (indx[j+1]-indx[j]) LE 1 then srm(j,*)=temp(indx[j],*) else $
        srm(j,*)=f_div( total(temp(indx[j]:indx[j+1]-1, *),1), indx[j+1]-indx[j])
  endfor

 if ~keyword_set(perbin) then srm= srm * reproduce(ectw, nph)
 

;RETURN
 
END
 


