;+
; Mission:
;   ASO-S/HXI
; Purpose:
;   Get visibility for user selected subcollimators
;   
; Inputs:
;   tarray  --  input time array of images
;   earray  --  input energy array of images
;   counts  --  counts data for given time intervals and energy ranges
;   counts_bg -- background data for given time interval and energy ranges
;   tot_counts  -- total counts, [n original time intervals]
;   tot_counts_bg -- total bkg counts, [n original time intervals]
;   xc,yc --  locations (in arcsec) of solar disk center, relative to the center of SAS FOV
;   cent_xy -- locations (in arcsec) of solar disk center, relative to the center of SAS FOV
;   det_id  -- ids for subcollimator, 0-91
;   
; Output:
;   vis_hxi -- HXI visibility array
; History:
;   Weichen, 2022/08/25,  created
;   Xiafanxiaoyu, 2022/09/17, if the selected detectors are not in pairs, returns an error.
;   Weichen, 2022/09/20,  add counts from the triple subcollimator
;   Yang Su, 2022/09/24,  changed the code to get vis for HXI, removed hxi_get_image_vis
;   Yang Su, 2023/04/03,  changed the way of getting 'dc' and 'total flux', under test
;   Wei Chen, 2023/04/18, check if det_id are in pairs
;   Yang Su, 2023/04/18,  now dc is the average of all modulated fluxes

FUNCTION hxi_get_vis, tarray, earray,counts,counts_bg,tot_counts,tot_counts_bg,xc=xc,yc=yc,cent_xy=cent_xy, $  ;0916
        det_id=det_id,no_pot_corr=no_pot_corr, vis_hxi
;counts,counts_bg,tot_counts,tot_counts_bg,xc=xc,yc=yc, $  ;0916
  
  default, no_pot_corr , 0
  
  if (size(counts))[0] gt 1 then ntbins = (size(counts))[2] else ntbins = 1
  u_moducts =  counts[det_id,*]
   
  ;u_modultm =  ltime [*,det_id]

  tot_cts= tot_counts
  bg_cts = tot_counts_bg

  dc= (tot_counts - tot_counts_bg)/4.

  info = hxi_grid_info()
  uv= info.uv 

   ;check if det_id are in pairs, enabled on 230418, wei chen
  indices = WHERE(det_id LT 88,  NCOMPLEMENT=count_triple) ;detoctors of 88,89,90 are paired of visibility  2022/09/20
  IF ~is_couple(det_id[indices]) THEN BEGIN   ;DETERMINE WHETHER THE SELECTED DETECTORS ARE PAIRED
    ;MESSAGE, 'Wrong set of detectors for vis imaging.'
    a = dialog_message('Wrong set of detectors for vis imaging.')
  ENDIF
  
  visids = fix(det_id)/2
  visids = visids[uniqo(visids < 44)]
;  visids = rem_elem(visids, 45) ;remove num 45 visibility if exist
  
  hxi_uv = uv;[*,visids]
  
;  IF TYPENAME(hxi_uv) EQ 'INT' THEN BEGIN
;    a = DIALOG_MESSAGE('The selected detectors need to be in pairs')
;    hxi_maps=-1
;    RETURN, hxi_maps
;  ENDIF
 
  nuv= 45 ;f_div( N_ELEMENTS(hxi_uv),2)

  vis= {hxi_vis}   ; hxi_visibility() ;
  vis.xyoffset= [xc,yc]
  
  if no_pot_corr eq 1 then begin
    tot_cts= total( tot_counts)
    bg_cts = total( tot_counts_bg )
    
    vis_hxi= REPLICATE(vis, nuv)

    vis_hxi.u= rebin(  REFORM( hxi_uv[0,*] ), nuv,/sample )
    vis_hxi.v= rebin(  REFORM( hxi_uv[1,*] ), nuv,/sample )

    spatfreq2       = ((vis_hxi.u^2 + vis_hxi.v^2)*2.33^2)
    scn             = ROUND(-ALOG(spatfreq2) / ALOG(3))
    vis_hxi.isc      = scn-1

    ;vis_hxi.isc=1
    vis_hxi.harm= INTARR(nuv)+1

    ;modu_counts= REFORM(u_moducts[*,i,j])
    modu_counts= REFORM(counts - counts_bg)>0.  ;2022/09/22
    ;modu_counts= total(counts,2 )  ;2022/11/09
    n_modu = (size(modu_counts))[1]
    dc_counts = dc
    ;dc_counts = average( modu_counts )   ;for test  ; 221109,YS
    dc_counts =  (total(tot_counts)-total(tot_counts_bg))/4.   ;for test  ; 240309,YS
    ;dc_counts = total( counts[-3:-1 ] - counts_bg[-3:-1],1 )/3.*4./4.   ;for test  ;;230403,yang su
    dc_counts_all = fltarr(n_modu)+ dc_counts ;  
    counts_cos = modu_counts[0:-2:2] - dc_counts_all[0:-2:2]
    counts_sin = modu_counts[1:*:2]  - dc_counts_all[1:*:2]

    ;if n_elements(counts_cos) great than counts_sin, means use tri_collimator detectors ; 2022/09/20
    IF n_modu MOD 2 THEN BEGIN
      temp = modu_counts[n_modu-3:*]

      last_modu_counts = hxi_cal_tri_collimator(temp)
      ;    counts_cos =  modu_counts[0:n_modu-3:2,*] - dc_counts_all[0:n_modu-3:2,*]
      ;    counts_sin =  modu_counts[1:n_modu-3:2,*] - dc_counts_all[1:n_modu-3:2,*]
      counts_cos[-1] =  modu_counts[-3] - dc_counts_all[-3]
      counts_sin[-1] =  last_modu_counts - dc_counts_all[-3]
    ENDIF

    vis_hxi.trange= anytim(tarray)
    vis_hxi.erange= earray
    vis_hxi.obsvis= COMPLEX(counts_cos, counts_sin);COMPLEX(counts_sin, counts_cos);COMPLEX(counts_cos, counts_sin)
    vis_hxi.totflux= fltarr(nuv)+ dc_counts*4.  ;230403,yang su;240112, YS
    vis_hxi.sigamp = 0.1 * MAX(ABS(vis_hxi.obsvis))
    ;vis_hxi.xyoffset = vis_hxi.xyoffset + transpose(rebin( cent_xy,2, n_elements(dc_counts),nuv,/sample ),[0,2,1]);yang, 220926
    ;vis_hxi.xyoffset =  rebin( reform(cent_xy[*,0]),2,nuv ,/sample) ;yang, 221109
    vis_hxi.xyoffset =  rebin( reform([xc,yc]),2,nuv ,/sample) ;yang su, 240108

    RETURN, reform(vis_hxi[visids])
  endif else begin
 
    vis_hxi= REPLICATE(vis, nuv,ntbins)

    vis_hxi.u= rebin(  REFORM( hxi_uv[0,*] ), nuv,ntbins,/sample )
    vis_hxi.v= rebin(  REFORM( hxi_uv[1,*] ), nuv,ntbins,/sample )
 
    spatfreq2       = ((vis_hxi.u^2 + vis_hxi.v^2)*2.33^2)
    scn             = ROUND(-ALOG(spatfreq2) / ALOG(3))
    vis_hxi.isc      = scn-1

    ;vis_hxi.isc=1
    vis_hxi.harm= INTARR(nuv,ntbins)+1

    ;modu_counts= REFORM(u_moducts[*,i,j])
    modu_counts= REFORM(counts - counts_bg)>0.  ;2022/09/22
    n_modu = (size(modu_counts))[1]
 
    ;dc_counts = total( counts[-3:-1,* ] - counts_bg[-3:-1,*],1 )/3.*4./4.   ;for test  ;;230403,yang su, 230418, comment out
    dc_counts = dc   ;average( modu_counts ,1) ;240309, YS
    dc_counts_all = transpose( rebin(dc_counts,n_elements(dc_counts),n_modu,/sample)  )
    counts_cos = modu_counts[0:-2:2,*]  - dc_counts_all[0:*:2,*]
    counts_sin = modu_counts[1:*:2,*]  - dc_counts_all[1:*:2,*]

    ;if n_elements(counts_cos) great than counts_sin, means use tri_collimator detectors ; 2022/09/20
    IF n_modu MOD 2 THEN BEGIN
      ;PRINT,'Notice that you are using a triple subcollimator detectors.'
      temp = modu_counts[n_modu-3:*,*]

      last_modu_counts = hxi_cal_tri_collimator(temp)
      ;    counts_cos =  modu_counts[0:n_modu-3:2,*] - dc_counts_all[0:n_modu-3:2,*]
      ;    counts_sin =  modu_counts[1:n_modu-3:2,*] - dc_counts_all[1:n_modu-3:2,*]
      counts_cos[-1,*] =  modu_counts[-3,*] - dc_counts_all[-3,*]
      counts_sin[-1,*] =  last_modu_counts - dc_counts_all[-3,*]
    ENDIF

    vis_hxi.trange= anytim(tarray)
    vis_hxi.erange= earray
    vis_hxi.obsvis= COMPLEX(counts_cos, counts_sin);COMPLEX(counts_sin, counts_cos);COMPLEX(counts_cos, counts_sin)
    vis_hxi.totflux=  transpose(rebin( dc_counts*4. ,n_elements(dc_counts),nuv,/sample ))   ;yang, 230403

    vis_hxi.sigamp = 0.1 * MAX(ABS(vis_hxi.obsvis))
    ;vis_hxi.xyoffset = vis_hxi.xyoffset + transpose(rebin( cent_xy,2, n_elements(dc_counts),nuv,/sample ),[0,2,1]);yang, 220926
    ;vis_hxi.xyoffset =  transpose(rebin( cent_xy,2, n_elements(dc_counts),nuv,/sample ),[0,2,1]);yang, 221017
    vis_hxi.xyoffset =  transpose( rebin(  ( cent_xy- rebin( reform(cent_xy[*,0])-[xc,yc],2, n_elements(dc_counts),/sample ) ), 2, n_elements(dc_counts),nuv,/sample ),[0,2,1]);yang, 240108

    RETURN, reform(vis_hxi[visids,*],n_elements(visids)*ntbins) ;yang, 220926
  endelse
  
 
END