;+
; Mission     : ASO-S/HXI
; Name        : hxi_mem_ge
; Category    : imaging
; Purpose:
; This is a wrapper around the MEM_GE function to obtain the MEM_GE image from HXI visibilities. 
; For more details about MEM_GE refer to Massa et al., "MEM_GE: A New Maximum Entropy Method for Image 
; Reconstruction from Solar X-Ray Visibilities", ApJ, 2020
;
; Inputs:
;   - visin: HXI visibility structure 
;
; Keyword inputs:
;   - imsize: two-element array containing the number of pixels of the reconstructed image (default, [129,129])
;   - pixel: two-element array containing the pixel size in arcsec (default, [1.,1.])
;   - total_flux: estimate of the total flux of the map to be used for constraining the sum of the pixel value 
;                 of the reconstruction. By default it is computed by the 'vis_estimate_flux' routine
;   - percent_lambda: value used to compute the regularization parameter as a percentage of a maximum value automatically 
;                     overestimated by the algorithm. Must be in the range [0.0001,0.2] (default, 0.02)      
;   - makemap: if set, a map structure is returned (default, 1)
;   - silent: if set, no text is printed by the MEM_GE routine (default, 0)
;
; Keyword outputs:
;   - map: if makemap is set, a map structure is returned. Otherwise, only the image matrix is returned through this keyword
;
; History:
; 18-mar-2023: Paolo Massa, created
;-

pro hxi_mem_ge, visin, imsize=imsize, pixel=pixel, total_flux=total_flux, percent_lambda=percent_lambda,$
                makemap=makemap, map=mem_ge_map, silent=silent, _extra=extra

  vis = visin
  default, imsize, [129, 129]
  default, pixel, [1.,1.]
  default, silent, 0
  default, percent_lambda, 0.02
  default, total_flux, vis_estimate_flux(vis, imsize[0]*pixel[0], silent=silent) 
  default, makemap, 1
  
  mapcenter = reform(vis.xyoffset)  
  mapcenter0 = reform(vis[0].xyoffset)
  
  ;; MEM_GE needs visibilities corresponding to mapcenter=(0,0). We apply a phase factor to the HXI visibilities
  phase_corr = -2 * !pi * (reform(mapcenter[0,*]) * vis.u + reform(mapcenter[1,*]) * vis.v )
  vis.obsvis = vis.obsvis * complex(cos(phase_corr), sin(phase_corr))
  
  mem_ge_map = mem_ge(vis, total_flux, imsize=imsize, pixel = pixel,$
    percent_lambda=percent_lambda, $
     silent=silent, makemap=makemap, _extra=extra) ;240309, YS, add input for imsize and pixel
                     
  if keyword_set(makemap) then begin
    mem_ge_map.xc = mapcenter0[0]
    mem_ge_map.yc = mapcenter0[1]
  endif

end
